from unittest.mock import ANY

from litestar.testing import TestClient


def test_create_user(user_data) -> None:
    from docs.examples.data_transfer_objects.factory.dto_data_usage import app

    with TestClient(app) as client:
        response = client.post("/users", json=user_data)

    assert response.status_code == 201
    assert response.json() == {"id": ANY, "name": "Mr Sunglass", "email": "mr.sunglass@example.com", "age": 30}



# ruff: noqa: F401
# pyright: reportUnusedImport=false
import importlib
import sys
from pathlib import Path
from typing import List, Union

import pytest


def purge_module(module_names: List[str], path: Union[str, Path]) -> None:
    for name in module_names:
        if name in sys.modules:
            del sys.modules[name]
    Path(importlib.util.cache_from_source(path)).unlink(missing_ok=True)  # type: ignore[arg-type]


def test_deprecated_htmx_request() -> None:
    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing HTMXDetails from 'litestar.contrib.htmx.request' is deprecated"
    ):
        from litestar.contrib.htmx.request import HTMXDetails

    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing HTMXDetails from 'litestar.contrib.htmx.request' is deprecated"
    ):
        from litestar.contrib.htmx.request import HTMXDetails


def test_deprecated_htmx_response() -> None:
    purge_module(["litestar.contrib.htmx.response"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing HTMXTemplate from 'litestar.contrib.htmx.response' is deprecated"
    ):
        from litestar.contrib.htmx.response import HTMXTemplate

    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing HXLocation from 'litestar.contrib.htmx.response' is deprecated"
    ):
        from litestar.contrib.htmx.response import HXLocation

    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing HXStopPolling from 'litestar.contrib.htmx.response' is deprecated"
    ):
        from litestar.contrib.htmx.response import HXStopPolling

    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing ClientRedirect from 'litestar.contrib.htmx.response' is deprecated"
    ):
        from litestar.contrib.htmx.response import ClientRedirect
    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing ClientRefresh from 'litestar.contrib.htmx.response' is deprecated"
    ):
        from litestar.contrib.htmx.response import ClientRefresh
    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing PushUrl from 'litestar.contrib.htmx.response' is deprecated"
    ):
        from litestar.contrib.htmx.response import PushUrl
    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing ReplaceUrl from 'litestar.contrib.htmx.response' is deprecated"
    ):
        from litestar.contrib.htmx.response import ReplaceUrl
    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(DeprecationWarning, match="importing Reswap from 'litestar.contrib.htmx.response' is deprecated"):
        from litestar.contrib.htmx.response import Reswap
    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing Retarget from 'litestar.contrib.htmx.response' is deprecated"
    ):
        from litestar.contrib.htmx.response import Retarget
    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing TriggerEvent from 'litestar.contrib.htmx.response' is deprecated"
    ):
        from litestar.contrib.htmx.response import TriggerEvent


def test_deprecated_htmx_types() -> None:
    purge_module(["litestar.contrib.htmx.types"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing HtmxHeaderType from 'litestar.contrib.htmx.types' is deprecated"
    ):
        from litestar.contrib.htmx.types import HtmxHeaderType

    purge_module(["litestar.contrib.htmx.types"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing TriggerEventType from 'litestar.contrib.htmx.types' is deprecated"
    ):
        from litestar.contrib.htmx.types import TriggerEventType

    purge_module(["litestar.contrib.htmx.request"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing LocationType from 'litestar.contrib.htmx.types' is deprecated"
    ):
        from litestar.contrib.htmx.types import LocationType



from typing import Dict

from litestar import Litestar, get


@get("/", sync_to_thread=False)
def index(param: str = "hello") -> Dict[str, str]:
    return {"param": param}


app = Litestar(route_handlers=[index])


# run: /
# run: /?param=john



# ruff: noqa: F401
from __future__ import annotations

import importlib
import sys
from importlib.util import cache_from_source
from pathlib import Path

import pytest


def purge_module(module_names: list[str], path: str | Path) -> None:
    for name in module_names:
        if name in sys.modules:
            del sys.modules[name]
    Path(cache_from_source(str(path))).unlink(missing_ok=True)


def test_deprecated_pydantic_imports() -> None:
    purge_module(["litestar.contrib.pydantic"], __file__)
    with pytest.warns(DeprecationWarning, match="importing PydanticDTO from 'litestar.contrib.pydantic' is deprecated"):
        from litestar.contrib.pydantic import PydanticDTO

    purge_module(["litestar.contrib.pydantic"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing PydanticInitPlugin from 'litestar.contrib.pydantic' is deprecated"
    ):
        from litestar.contrib.pydantic import PydanticInitPlugin

    purge_module(["litestar.contrib.pydantic"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing PydanticSchemaPlugin from 'litestar.contrib.pydantic' is deprecated"
    ):
        from litestar.contrib.pydantic import PydanticSchemaPlugin

    purge_module(["litestar.contrib.pydantic"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing PydanticPlugin from 'litestar.contrib.pydantic' is deprecated"
    ):
        from litestar.contrib.pydantic import PydanticPlugin

    purge_module(["litestar.contrib.pydantic"], __file__)
    with pytest.warns(
        DeprecationWarning, match="importing PydanticDIPlugin from 'litestar.contrib.pydantic' is deprecated"
    ):
        from litestar.contrib.pydantic import PydanticDIPlugin


def test_deprecated_pydantic_dto_factory_imports() -> None:
    purge_module(["litestar.contrib.pydantic.pydantic_dto_factory"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing PydanticDTO from 'litestar.contrib.pydantic' is deprecated",
    ):
        from litestar.contrib.pydantic import PydanticDTO


def test_deprecated_pydantic_init_plugin_imports() -> None:
    purge_module(["litestar.contrib.pydantic.pydantic_init_plugin"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing PydanticInitPlugin from 'litestar.contrib.pydantic' is deprecated",
    ):
        from litestar.contrib.pydantic import PydanticInitPlugin


def test_deprecated_pydantic_schema_plugin_imports() -> None:
    purge_module(["litestar.contrib.pydantic.pydantic_schema_plugin"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing PydanticSchemaPlugin from 'litestar.contrib.pydantic' is deprecated",
    ):
        from litestar.contrib.pydantic import PydanticSchemaPlugin


def test_deprecated_pydantic_di_plugin_imports() -> None:
    purge_module(["litestar.contrib.pydantic"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing PydanticDIPlugin from 'litestar.contrib.pydantic' is deprecated",
    ):
        from litestar.contrib.pydantic import PydanticDIPlugin


def test_deprecated_pydantic_utils_imports() -> None:
    purge_module(["litestar.contrib.pydantic.utils"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing get_model_info from 'litestar.contrib.pydantic.utils' is deprecated",
    ):
        from litestar.contrib.pydantic.utils import get_model_info

    purge_module(["litestar.contrib.pydantic.utils"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing is_pydantic_constrained_field from 'litestar.contrib.pydantic.utils' is deprecated",
    ):
        from litestar.contrib.pydantic.utils import is_pydantic_constrained_field

    purge_module(["litestar.contrib.pydantic.utils"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing is_pydantic_model_class from 'litestar.contrib.pydantic.utils' is deprecated",
    ):
        from litestar.contrib.pydantic.utils import is_pydantic_model_class

    purge_module(["litestar.contrib.pydantic.utils"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing is_pydantic_undefined from 'litestar.contrib.pydantic.utils' is deprecated",
    ):
        from litestar.contrib.pydantic.utils import is_pydantic_undefined

    purge_module(["litestar.contrib.pydantic.utils"], __file__)
    with pytest.warns(
        DeprecationWarning,
        match="importing is_pydantic_v2 from 'litestar.contrib.pydantic.utils' is deprecated",
    ):
        from litestar.contrib.pydantic.utils import is_pydantic_v2



from typing import Any, Dict

from litestar import Litestar, Request
from litestar.plugins.prometheus import PrometheusConfig, PrometheusController


# We can modify the path of our custom handler and override the metrics format by subclassing the PrometheusController.
class CustomPrometheusController(PrometheusController):
    path = "/custom-path"
    openmetrics_format = True


# Let's assume this as our extra custom labels which we want our metrics to have.
# The values can be either a string or a callable that returns a string.
def custom_label_callable(request: Request[Any, Any, Any]) -> str:
    return "v2.0"


extra_labels = {
    "version_no": custom_label_callable,
    "location": "earth",
}

# Customizing the buckets for the histogram.
buckets = [0.1, 0.2, 0.3, 0.4, 0.5]


# Adding exemplars to the metrics.
# Note that this supported only in openmetrics format.
def custom_exemplar(request: Request[Any, Any, Any]) -> Dict[str, str]:
    return {"trace_id": "1234"}


# Creating the instance of PrometheusConfig with our own custom options.
# The given options are not necessary, you can use the default ones
# as well by just creating a raw instance PrometheusConfig()
prometheus_config = PrometheusConfig(
    app_name="litestar-example",
    prefix="litestar",
    labels=extra_labels,
    buckets=buckets,  # pyright: ignore[reportArgumentType]
    exemplars=custom_exemplar,
    excluded_http_methods=["POST"],
)


# Creating the litestar app instance with our custom PrometheusConfig and PrometheusController.
app = Litestar(route_handlers=[CustomPrometheusController], middleware=[prometheus_config.middleware])



from __future__ import annotations

from dataclasses import dataclass, field
from typing import List
from uuid import UUID, uuid4

from litestar import Controller, delete, get, post, put
from litestar.app import Litestar
from litestar.dto import DataclassDTO
from litestar.dto.config import DTOConfig


@dataclass
class User:
    name: str
    email: str
    age: int
    id: UUID = field(default_factory=uuid4)


class UserWriteDTO(DataclassDTO[User]):
    config = DTOConfig(exclude={"id"})


class UserReadDTO(DataclassDTO[User]): ...


class UserController(Controller):
    dto = UserWriteDTO
    return_dto = UserReadDTO

    @post("/", sync_to_thread=False)
    def create_user(self, data: User) -> User:
        return data

    @get("/", sync_to_thread=False)
    def get_users(self) -> List[User]:
        return [User(name="Mr Sunglass", email="mr.sunglass@example.com", age=30)]

    @get("/{user_id:uuid}", sync_to_thread=False)
    def get_user(self, user_id: UUID) -> User:
        return User(id=user_id, name="Mr Sunglass", email="mr.sunglass@example.com", age=30)

    @put("/{user_id:uuid}", sync_to_thread=False)
    def update_user(self, data: User) -> User:
        return data

    @delete("/{user_id:uuid}", return_dto=None, sync_to_thread=False)
    def delete_user(self, user_id: UUID) -> None:
        return None


app = Litestar([UserController])



from pathlib import Path

from litestar import Litestar
from litestar.middleware.session.server_side import ServerSideSessionConfig
from litestar.stores.file import FileStore

app = Litestar(
    middleware=[ServerSideSessionConfig().middleware],
    stores={"sessions": FileStore(path=Path("session_data"))},
)



from typing import Dict

from litestar import Litestar, get
from litestar.openapi.config import OpenAPIConfig


@get("/")
def hello_world() -> Dict[str, str]:
    return {"message": "Hello World"}


app = Litestar(
    route_handlers=[hello_world],
    openapi_config=OpenAPIConfig(
        title="My API",
        description="This is the description of my API",
        version="0.1.0",
        path="/docs",
    ),
)



from litestar import Litestar, Request
from litestar.config.response_cache import ResponseCacheConfig


def key_builder(request: Request) -> str:
    return request.url.path + request.headers.get("my-header", "")


app = Litestar([], response_cache_config=ResponseCacheConfig(key_builder=key_builder))



from __future__ import annotations

import contextlib
from itertools import chain
from typing import TYPE_CHECKING, Any

from msgspec.msgpack import decode as _decode_msgpack_plain

from litestar.datastructures.multi_dicts import FormMultiDict
from litestar.enums import HttpMethod, MediaType, ScopeType
from litestar.exceptions import ClientException, ImproperlyConfiguredException, SerializationException
from litestar.handlers.http_handlers import HTTPRouteHandler
from litestar.response import Response
from litestar.routes.base import BaseRoute
from litestar.status_codes import HTTP_204_NO_CONTENT
from litestar.types.empty import Empty
from litestar.utils.scope.state import ScopeState

if TYPE_CHECKING:
    from litestar._kwargs import KwargsModel
    from litestar.connection import Request
    from litestar.types import ASGIApp, HTTPScope, Method, Receive, Scope, Send


class HTTPRoute(BaseRoute):
    """An HTTP route, capable of handling multiple ``HTTPRouteHandler``\\ s."""  # noqa: D301

    __slots__ = (
        "route_handler_map",
        "route_handlers",
    )

    def __init__(
        self,
        *,
        path: str,
        route_handlers: list[HTTPRouteHandler],
    ) -> None:
        """Initialize ``HTTPRoute``.

        Args:
            path: The path for the route.
            route_handlers: A list of :class:`~.handlers.HTTPRouteHandler`.
        """
        methods = list(chain.from_iterable([route_handler.http_methods for route_handler in route_handlers]))
        if "OPTIONS" not in methods:
            methods.append("OPTIONS")
            options_handler = self.create_options_handler(path)
            options_handler.owner = route_handlers[0].owner
            route_handlers.append(options_handler)

        self.route_handlers = route_handlers
        self.route_handler_map: dict[Method, tuple[HTTPRouteHandler, KwargsModel]] = {}

        super().__init__(
            methods=methods,
            path=path,
            scope_type=ScopeType.HTTP,
            handler_names=[route_handler.handler_name for route_handler in self.route_handlers],
        )

    async def handle(self, scope: HTTPScope, receive: Receive, send: Send) -> None:  # type: ignore[override]
        """ASGI app that creates a Request from the passed in args, determines which handler function to call and then
        handles the call.

        Args:
            scope: The ASGI connection scope.
            receive: The ASGI receive function.
            send: The ASGI send function.

        Returns:
            None
        """
        route_handler, parameter_model = self.route_handler_map[scope["method"]]
        request: Request[Any, Any, Any] = route_handler.resolve_request_class()(scope=scope, receive=receive, send=send)

        if route_handler.resolve_guards():
            await route_handler.authorize_connection(connection=request)

        try:
            response = await self._get_response_for_request(
                scope=scope, request=request, route_handler=route_handler, parameter_model=parameter_model
            )

            await response(scope, receive, send)

            if after_response_handler := route_handler.resolve_after_response():
                await after_response_handler(request)
        finally:
            if (form_data := ScopeState.from_scope(scope).form) is not Empty:
                await FormMultiDict.from_form_data(form_data).close()

    def create_handler_map(self) -> None:
        """Parse the ``router_handlers`` of this route and return a mapping of
        http- methods and route handlers.
        """
        for route_handler in self.route_handlers:
            kwargs_model = route_handler.create_kwargs_model(path_parameters=self.path_parameters)
            for http_method in route_handler.http_methods:
                if self.route_handler_map.get(http_method):
                    raise ImproperlyConfiguredException(
                        f"Handler already registered for path {self.path!r} and http method {http_method}"
                    )
                self.route_handler_map[http_method] = (route_handler, kwargs_model)

    async def _get_response_for_request(
        self,
        scope: Scope,
        request: Request[Any, Any, Any],
        route_handler: HTTPRouteHandler,
        parameter_model: KwargsModel,
    ) -> ASGIApp:
        """Return a response for the request.

        If caching is enabled and a response exist in the cache, the cached response will be returned.
        If caching is enabled and a response does not exist in the cache, the newly created
        response will be cached.

        Args:
            scope: The Request's scope
            request: The Request instance
            route_handler: The HTTPRouteHandler instance
            parameter_model: The Handler's KwargsModel

        Returns:
            An instance of Response or a compatible ASGIApp or a subclass of it
        """
        if route_handler.cache and (
            response := await self._get_cached_response(request=request, route_handler=route_handler)
        ):
            return response

        return await self._call_handler_function(
            scope=scope, request=request, parameter_model=parameter_model, route_handler=route_handler
        )

    async def _call_handler_function(  # type: ignore[return]
        self, scope: Scope, request: Request, parameter_model: KwargsModel, route_handler: HTTPRouteHandler
    ) -> ASGIApp:  # pyright: ignore[reportGeneralTypeIssues]
        """Call the before request handlers, retrieve any data required for the route handler, and call the route
        handler's ``to_response`` method.

        This is wrapped in a try except block - and if an exception is raised,
        it tries to pass it to an appropriate exception handler - if defined.
        """
        response_data: Any = None

        if before_request_handler := route_handler.resolve_before_request():
            response_data = await before_request_handler(request)

        # create and enter an AsyncExit stack as we may or may not have a
        # 'DependencyCleanupGroup' to enter and exit
        stack = contextlib.AsyncExitStack()

        # mypy cannot infer that 'stack' never swallows exceptions, therefore it thinks
        # this method is potentially missing a 'return' statement
        async with stack:
            if not response_data:
                parsed_kwargs: dict[str, Any] = {}

                if parameter_model.has_kwargs and route_handler.signature_model:
                    try:
                        kwargs = await parameter_model.to_kwargs(connection=request)
                    except SerializationException as e:
                        raise ClientException(str(e)) from e

                    if kwargs.get("data") is Empty:
                        del kwargs["data"]

                    if parameter_model.dependency_batches:
                        cleanup_group = await parameter_model.resolve_dependencies(request, kwargs)
                        await stack.enter_async_context(cleanup_group)

                    parsed_kwargs = route_handler.signature_model.parse_values_from_connection_kwargs(
                        connection=request, kwargs=kwargs
                    )

                response_data = (
                    route_handler.fn(**parsed_kwargs)
                    if route_handler.has_sync_callable
                    else await route_handler.fn(**parsed_kwargs)
                )

            return await route_handler.to_response(app=scope["litestar_app"], data=response_data, request=request)

    @staticmethod
    async def _get_cached_response(request: Request, route_handler: HTTPRouteHandler) -> ASGIApp | None:
        """Retrieve and un-pickle the cached response, if existing.

        Args:
            request: The :class:`Request <litestar.connection.Request>` instance
            route_handler: The :class:`~.handlers.HTTPRouteHandler` instance

        Returns:
            A cached response instance, if existing.
        """

        cache_config = request.app.response_cache_config
        cache_key = (route_handler.cache_key_builder or cache_config.key_builder)(request)
        store = cache_config.get_store_from_app(request.app)

        if not (cached_response_data := await store.get(key=cache_key)):
            return None

        # we use the regular msgspec.msgpack.decode here since we don't need any of
        # the added decoders
        messages = _decode_msgpack_plain(cached_response_data)

        async def cached_response(scope: Scope, receive: Receive, send: Send) -> None:
            ScopeState.from_scope(scope).is_cached = True
            for message in messages:
                await send(message)

        return cached_response

    def create_options_handler(self, path: str) -> HTTPRouteHandler:
        """Args:
            path: The route path

        Returns:
            An HTTP route handler for OPTIONS requests.
        """

        def options_handler(scope: Scope) -> Response:
            """Handler function for OPTIONS requests.

            Args:
                scope: The ASGI Scope.

            Returns:
                Response
            """
            return Response(
                content=None,
                status_code=HTTP_204_NO_CONTENT,
                headers={"Allow": ", ".join(sorted(self.methods))},  # pyright: ignore
                media_type=MediaType.TEXT,
            )

        return HTTPRouteHandler(
            path=path,
            http_method=[HttpMethod.OPTIONS],
            include_in_schema=False,
            sync_to_thread=False,
        )(options_handler)



import pathlib
import socket
import subprocess
import time
from contextlib import asynccontextmanager, contextmanager
from typing import AsyncIterator, Iterator

import httpx


class StartupError(RuntimeError):
    pass


def _get_available_port() -> int:
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
        # Bind to a free port provided by the host
        try:
            sock.bind(("localhost", 0))
        except OSError as e:  # pragma: no cover
            raise StartupError("Could not find an open port") from e
        else:
            port: int = sock.getsockname()[1]
            return port


@contextmanager
def run_app(
    workdir: pathlib.Path, app: str, capture_output: bool, retry_count: int = 100, retry_timeout: int = 1
) -> Iterator[str]:
    """Launch a litestar application in a subprocess with a random available port.

    Args:
        workdir: Path to working directory where run command will be executed
        app: Path to Litestar application, e.g.: "my_app:application"
        capture_output: Whether to capture output from the subprocess in the main process stdout/stderr
        retry_count: Number of retries to wait for the application to start
        retry_timeout: Timeout in seconds to wait between retries

    Raises:
        StartupError: If the application fails to start with given retry count and timeout
    """
    port = _get_available_port()
    output = None if capture_output else subprocess.DEVNULL
    with subprocess.Popen(
        args=["litestar", "--app", app, "run", "--port", str(port)],
        stderr=output,
        stdout=output,
        cwd=workdir,
    ) as proc:
        url = f"http://127.0.0.1:{port}"
        application_started = False

        for _ in range(retry_count):
            try:
                httpx.get(url, timeout=0.1)
                application_started = True
                break
            except httpx.TransportError:
                time.sleep(retry_timeout)

        if not application_started:
            proc.kill()
            raise StartupError("Application failed to start")

        yield url
        proc.kill()


@asynccontextmanager
async def subprocess_async_client(
    workdir: pathlib.Path, app: str, capture_output: bool = True
) -> AsyncIterator[httpx.AsyncClient]:
    """Provides an async httpx client for a litestar app launched in a subprocess.

    Args:
        workdir: Path to the directory in which the app module resides.
        app: Uvicorn app string that can be resolved in the provided working directory, e.g.: "app:app"
        capture_output: Whether to capture output from the subprocess in the main process stdout/stderr
    """
    with run_app(workdir=workdir, app=app, capture_output=capture_output) as url:
        async with httpx.AsyncClient(base_url=url) as client:
            yield client


@contextmanager
def subprocess_sync_client(workdir: pathlib.Path, app: str, capture_output: bool = True) -> Iterator[httpx.Client]:
    """Provides a sync httpx client for a litestar app launched in a subprocess.

    Args:
        workdir: Path to the directory in which the app module resides.
        app: Uvicorn app string that can be resolved in the provided working directory, e.g.: "app:app"
        capture_output: Whether to capture output from the subprocess in the main process stdout/stderr
    """
    with run_app(workdir=workdir, app=app, capture_output=capture_output) as url, httpx.Client(base_url=url) as client:
        yield client



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from litestar.plugins import InitPlugin
from litestar.plugins.pydantic.dto import PydanticDTO
from litestar.plugins.pydantic.plugins.di import PydanticDIPlugin
from litestar.plugins.pydantic.plugins.init import PydanticInitPlugin
from litestar.plugins.pydantic.plugins.schema import PydanticSchemaPlugin

if TYPE_CHECKING:
    from pydantic import BaseModel
    from pydantic.v1 import BaseModel as BaseModelV1

    from litestar.config.app import AppConfig
    from litestar.types.serialization import PydanticV1FieldsListType, PydanticV2FieldsListType

__all__ = (
    "PydanticDIPlugin",
    "PydanticDTO",
    "PydanticInitPlugin",
    "PydanticPlugin",
    "PydanticSchemaPlugin",
)


def _model_dump(
    model: BaseModel | BaseModelV1,
    *,
    by_alias: bool = False,
    round_trip: bool = False,
) -> dict[str, Any]:
    return (
        model.model_dump(mode="json", by_alias=by_alias, round_trip=round_trip)  # pyright: ignore
        if hasattr(model, "model_dump")
        else {k: v.decode() if isinstance(v, bytes) else v for k, v in model.dict(by_alias=by_alias).items()}
    )


def _model_dump_json(
    model: BaseModel | BaseModelV1,
    by_alias: bool = False,
    round_trip: bool = False,
) -> str:
    return (
        model.model_dump_json(by_alias=by_alias, round_trip=round_trip)  # pyright: ignore
        if hasattr(model, "model_dump_json")
        else model.json(by_alias=by_alias)  # pyright: ignore
    )


class PydanticPlugin(InitPlugin):
    """A plugin that provides Pydantic integration."""

    __slots__ = (
        "exclude",
        "exclude_defaults",
        "exclude_none",
        "exclude_unset",
        "include",
        "prefer_alias",
        "round_trip",
        "validate_strict",
    )

    def __init__(
        self,
        exclude: PydanticV1FieldsListType | PydanticV2FieldsListType | None = None,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
        exclude_unset: bool = False,
        include: PydanticV1FieldsListType | PydanticV2FieldsListType | None = None,
        prefer_alias: bool = False,
        validate_strict: bool = False,
        round_trip: bool = False,
    ) -> None:
        """Pydantic Plugin to support serialization / validation of Pydantic types / models

        :param exclude: Fields to exclude during serialization
        :param exclude_defaults: Fields to exclude during serialization when they are set to their default value
        :param exclude_none: Fields to exclude during serialization when they are set to ``None``
        :param exclude_unset: Fields to exclude during serialization when they arenot set
        :param include: Fields to exclude during serialization
        :param prefer_alias: Use the ``by_alias=True`` flag when dumping models
        :param validate_strict: Use ``strict=True`` when calling ``.model_validate`` on Pydantic 2.x models
        :param round_trip: use ``round_trip=True`` when calling ``.model_dump``
          and ``.model_dump_json`` on Pydantic 2.x models
        """
        self.exclude = exclude
        self.exclude_defaults = exclude_defaults
        self.exclude_none = exclude_none
        self.exclude_unset = exclude_unset
        self.include = include
        self.prefer_alias = prefer_alias
        self.validate_strict = validate_strict
        self.round_trip = round_trip

    def on_app_init(self, app_config: AppConfig) -> AppConfig:
        """Configure application for use with Pydantic.

        Args:
            app_config: The :class:`AppConfig <.config.app.AppConfig>` instance.
        """
        app_config.plugins.extend(
            [
                PydanticInitPlugin(
                    exclude=self.exclude,
                    exclude_defaults=self.exclude_defaults,
                    exclude_none=self.exclude_none,
                    exclude_unset=self.exclude_unset,
                    include=self.include,
                    prefer_alias=self.prefer_alias,
                    validate_strict=self.validate_strict,
                    round_trip=self.round_trip,
                ),
                PydanticSchemaPlugin(prefer_alias=self.prefer_alias),
                PydanticDIPlugin(),
            ]
        )
        return app_config



from unittest.mock import MagicMock

import pytest

from litestar import Controller, Litestar, MediaType, asgi
from litestar.enums import ScopeType
from litestar.exceptions import LitestarWarning
from litestar.response.base import ASGIResponse
from litestar.status_codes import HTTP_200_OK
from litestar.testing import create_test_client
from litestar.types import ASGIApp, Receive, Scope, Send


def test_handle_asgi() -> None:
    @asgi(path="/")
    async def root_asgi_handler(scope: Scope, receive: Receive, send: Send) -> None:
        assert scope["type"] == ScopeType.HTTP
        assert scope["method"] == "GET"
        response = ASGIResponse(body=b"Hello World", media_type=MediaType.TEXT)
        await response(scope, receive, send)

    class MyController(Controller):
        path = "/asgi"

        @asgi()
        async def root_asgi_handler(self, scope: Scope, receive: Receive, send: Send) -> None:
            assert scope["type"] == ScopeType.HTTP
            assert scope["method"] == "GET"
            response = ASGIResponse(body=b"Hello World", media_type=MediaType.TEXT)
            await response(scope, receive, send)

    with create_test_client([root_asgi_handler, MyController]) as client:
        response = client.get("/")
        assert response.status_code == HTTP_200_OK
        assert response.text == "Hello World"
        response = client.get("/asgi")
        assert response.status_code == HTTP_200_OK
        assert response.text == "Hello World"


def test_asgi_signature_namespace() -> None:
    class MyController(Controller):
        path = "/asgi"
        signature_namespace = {"b": Receive}

        @asgi(signature_namespace={"c": Send})
        async def root_asgi_handler(
            self,
            scope: "a",  # type:ignore[name-defined]  # noqa: F821
            receive: "b",  # type:ignore[name-defined]  # noqa: F821
            send: "c",  # type:ignore[name-defined]  # noqa: F821
        ) -> None:
            await ASGIResponse(body=scope["path"].encode(), media_type=MediaType.TEXT)(scope, receive, send)

    with create_test_client([MyController], signature_namespace={"a": Scope}) as client:
        response = client.get("/asgi")
        assert response.status_code == HTTP_200_OK
        assert response.text == "/asgi"


def test_copy_scope_not_set_warns_on_modification() -> None:
    @asgi(is_mount=True)
    async def handler(scope: "Scope", receive: "Receive", send: "Send") -> None:
        scope["foo"] = ""  # type: ignore[typeddict-unknown-key]
        await ASGIResponse()(scope, receive, send)

    with create_test_client([handler]) as client:
        with pytest.warns(LitestarWarning, match="modified 'scope' with 'copy_scope' set to 'None'"):
            response = client.get("/")
        assert response.status_code == HTTP_200_OK


@pytest.mark.parametrize("copy_scope, expected_value", [(True, None), (False, "foo")])
def test_copy_scope(copy_scope: bool, expected_value: "str | None") -> None:
    mock = MagicMock()

    def middleware_factory(app: Litestar) -> ASGIApp:
        async def middleware(scope: "Scope", receive: "Receive", send: "Send") -> None:
            await app(scope, receive, send)
            mock(scope.get("foo"))

        return middleware

    @asgi(is_mount=True, copy_scope=copy_scope)
    async def handler(scope: "Scope", receive: "Receive", send: "Send") -> None:
        scope["foo"] = "foo"  # type: ignore[typeddict-unknown-key]
        await ASGIResponse()(scope, receive, send)

    with create_test_client([handler], middleware=[middleware_factory]) as client:
        client.get("/")

    mock.assert_called_once_with(expected_value)



# ruff: noqa: F401
# pyright: reportUnusedImport=false
from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.utils import warn_deprecation

__all__ = (
    "AlembicAsyncConfig",
    "AsyncSessionConfig",
    "SQLAlchemyAsyncConfig",
    "autocommit_before_send_handler",
    "default_before_send_handler",
)


def __getattr__(attr_name: str) -> object:
    if attr_name in __all__:
        if attr_name == "SQLAlchemyAsyncConfig":
            from advanced_alchemy.extensions.litestar.plugins.init.config.asyncio import (
                SQLAlchemyAsyncConfig as _SQLAlchemyAsyncConfig,
            )
            from sqlalchemy.ext.asyncio import AsyncEngine

            from litestar.contrib.sqlalchemy.plugins.init.config.compat import (
                _CreateEngineMixin,  # pyright: ignore[reportPrivateUsage]
            )

            class SQLAlchemyAsyncConfig(_SQLAlchemyAsyncConfig, _CreateEngineMixin[AsyncEngine]): ...

            module = "litestar.plugins.sqlalchemy"
            value = globals()[attr_name] = SQLAlchemyAsyncConfig
        elif attr_name in {"default_before_send_handler", "autocommit_before_send_handler"}:
            module = "litestar.plugins.sqlalchemy.plugins.init.config.asyncio"
            from advanced_alchemy.extensions.litestar.plugins.init.config.asyncio import (
                autocommit_before_send_handler,
                default_before_send_handler,
            )

            value = globals()[attr_name] = locals()[attr_name]
        else:
            module = "litestar.plugins.sqlalchemy"
            from advanced_alchemy.extensions.litestar import (
                AlembicAsyncConfig,
                AsyncSessionConfig,
            )

            value = globals()[attr_name] = locals()[attr_name]

        warn_deprecation(
            deprecated_name=f"litestar.contrib.sqlalchemy.plugins.init.config.asyncio.{attr_name}",
            version="2.12",
            kind="import",
            removal_in="3.0",
            info=f"importing {attr_name} from 'litestar.contrib.sqlalchemy.plugins.init.config.asyncio' is deprecated, please "
            f"import it from '{module}' instead",
        )
        value = globals()[attr_name] = locals()[attr_name]
        return value

    raise AttributeError(f"module {__name__!r} has no attribute {attr_name!r}")  # pragma: no cover


if TYPE_CHECKING:
    from advanced_alchemy.extensions.litestar import (
        AlembicAsyncConfig,
        AsyncSessionConfig,
        SQLAlchemyAsyncConfig,
    )
    from advanced_alchemy.extensions.litestar.plugins.init.config.asyncio import (
        autocommit_before_send_handler,
        default_before_send_handler,
    )



from pydantic import BaseModel

from litestar import Litestar, get

USER_DB = {1: {"id": 1, "name": "John Doe"}}


class User(BaseModel):
    id: int
    name: str


@get("/user/{user_id:int}", sync_to_thread=False)
def get_user(user_id: int) -> User:
    return User.model_validate(USER_DB[user_id])


app = Litestar(route_handlers=[get_user])



from litestar import Litestar, Request, get
from litestar.datastructures import ResponseHeader


@get("/")
async def hello_world1(request: Request) -> None:
    request.logger.info("inside request")
    return


app1 = Litestar(
    route_handlers=[hello_world1],
    response_headers=[ResponseHeader(name="X-Version", value="ABCD", description="Test")],
)


def test_included_header_fields() -> None:
    # https://github.com/litestar-org/litestar/issues/3416

    assert app1.openapi_schema.to_schema()["paths"]["/"]["get"]["responses"]["200"]["headers"] == {
        "X-Version": {
            "deprecated": False,
            "description": "Test",
            "required": False,
            "schema": {"type": "string"},
        }
    }



from .config import PrometheusConfig
from .controller import PrometheusController
from .middleware import PrometheusMiddleware

__all__ = ("PrometheusConfig", "PrometheusController", "PrometheusMiddleware")



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, Set

if TYPE_CHECKING:
    from collections import deque
    from collections.abc import Collection
    from datetime import date, datetime, time
    from decimal import Decimal
    from enum import Enum, IntEnum
    from ipaddress import (
        IPv4Address,
        IPv4Interface,
        IPv4Network,
        IPv6Address,
        IPv6Interface,
        IPv6Network,
    )
    from pathlib import Path, PurePath
    from re import Pattern
    from uuid import UUID

    from msgspec import Raw, Struct
    from msgspec.msgpack import Ext
    from typing_extensions import TypeAlias

    from litestar.types import DataclassProtocol, TypedDictClass

    try:
        from pydantic import BaseModel
        from pydantic.main import IncEx
        from pydantic.typing import AbstractSetIntStr, MappingIntStrAny
    except ImportError:
        BaseModel = Any  # type: ignore[assignment, misc]
        IncEx = Any  # type: ignore[misc]
        AbstractSetIntStr = Any
        MappingIntStrAny = Any

    try:
        from attrs import AttrsInstance
    except ImportError:
        AttrsInstance = Any  # type: ignore[assignment, misc]

__all__ = (
    "DataContainerType",
    "EncodableBuiltinCollectionType",
    "EncodableBuiltinType",
    "EncodableMsgSpecType",
    "EncodableStdLibIPType",
    "EncodableStdLibType",
    "LitestarEncodableType",
)

EncodableBuiltinType: TypeAlias = "None | bool | int | float | str | bytes | bytearray"
EncodableBuiltinCollectionType: TypeAlias = "list | tuple | set | frozenset | dict | Collection"
EncodableStdLibType: TypeAlias = (
    "date | datetime | deque | time | UUID | Decimal | Enum | IntEnum | DataclassProtocol | Path | PurePath | Pattern"
)
EncodableStdLibIPType: TypeAlias = (
    "IPv4Address | IPv4Interface | IPv4Network | IPv6Address | IPv6Interface | IPv6Network"
)
EncodableMsgSpecType: TypeAlias = "Ext | Raw | Struct"
LitestarEncodableType: TypeAlias = "EncodableBuiltinType | EncodableBuiltinCollectionType | EncodableStdLibType | EncodableStdLibIPType | EncodableMsgSpecType | BaseModel | AttrsInstance"  # pyright: ignore
DataContainerType: TypeAlias = "Struct | BaseModel | AttrsInstance | TypedDictClass | DataclassProtocol"  # pyright: ignore
PydanticV2FieldsListType: TypeAlias = "Set[int] | Set[str] | Dict[int, Any] | Dict[str, Any]"
PydanticV1FieldsListType: TypeAlias = "IncEx | AbstractSetIntStr | MappingIntStrAny"  # pyright: ignore



from litestar import Litestar, get
from litestar.config.response_cache import CACHE_FOREVER


@get("/cached", cache=True)
async def my_cached_handler() -> str:
    return "cached"


@get("/cached-seconds", cache=120)  # seconds
async def my_cached_handler_seconds() -> str:
    return "cached for 120 seconds"


@get("/cached-forever", cache=CACHE_FOREVER)
async def my_cached_handler_forever() -> str:
    return "cached forever"


app = Litestar(
    [my_cached_handler, my_cached_handler_seconds, my_cached_handler_forever],
)



from typing import Dict

from litestar import Litestar, MediaType, Response, get


async def after_request(response: Response) -> Response:
    if response.media_type == MediaType.TEXT:
        return Response({"message": response.content})
    return response


@get("/hello")
async def hello() -> str:
    return "Hello, world"


@get("/goodbye")
async def goodbye() -> Dict[str, str]:
    return {"message": "Goodbye"}


app = Litestar(route_handlers=[hello, goodbye], after_request=after_request)


# run: /hello
# run: /goodbye



from typing import Any, Dict, Literal, Optional
from uuid import UUID, uuid4

from pydantic import BaseModel, EmailStr, SecretStr

from litestar import Litestar, Request, get, post
from litestar.connection import ASGIConnection
from litestar.exceptions import NotAuthorizedException
from litestar.middleware.session.server_side import ServerSideSessionBackend, ServerSideSessionConfig
from litestar.openapi.config import OpenAPIConfig
from litestar.security.session_auth import SessionAuth
from litestar.stores.memory import MemoryStore


# Let's assume we have a User model that is a pydantic model.
# This though is not required - we need some sort of user class -
# but it can be any arbitrary value, e.g. an SQLAlchemy model,
# a representation of a MongoDB  etc.
class User(BaseModel):
    id: UUID
    name: str
    email: EmailStr


# we also have pydantic types for two different
# kinds of POST request bodies: one for creating
# a user, e.g. "sign-up", and the other for logging
# an existing user in.
class UserCreatePayload(BaseModel):
    name: str
    email: EmailStr
    password: SecretStr


class UserLoginPayload(BaseModel):
    email: EmailStr
    password: SecretStr


MOCK_DB: Dict[str, User] = {}
memory_store = MemoryStore()


# The SessionAuth class requires a handler callable
# that takes the session dictionary, and returns the
# 'User' instance correlating to it.
#
# The session dictionary itself is a value the user decides
# upon. So for example, it might be a simple dictionary
# that holds a user id, for example: { "id": "abcd123" }
#
# Note: The callable can be either sync or async - both will work.
async def retrieve_user_handler(
    session: Dict[str, Any], connection: "ASGIConnection[Any, Any, Any, Any]"
) -> Optional[User]:
    return MOCK_DB.get(user_id) if (user_id := session.get("user_id")) else None


@post("/login")
async def login(data: UserLoginPayload, request: "Request[Any, Any, Any]") -> User:
    # we received log-in data via post.
    # our login handler should retrieve from persistence (a db etc.)
    # the user data and verify that the login details
    # are correct. If we are using passwords, we should check that
    # the password hashes match etc. We will simply assume that we
    # have done all of that we now have a user value:
    user_id = await memory_store.get(data.email)

    if not user_id:
        raise NotAuthorizedException
    user_id = user_id.decode("utf-8")

    # once verified we can create a session.
    # to do this we simply need to call the Litestar
    # 'Request.set_session' method, which accepts either dictionaries
    # or pydantic models. In our case, we can simply record a
    # simple dictionary with the user ID value:
    request.set_session({"user_id": user_id})

    # you can do whatever we want here. In this case, we will simply return the user data:
    return MOCK_DB[user_id]


@post("/signup")
async def signup(data: UserCreatePayload, request: Request[Any, Any, Any]) -> User:
    # this is similar to the login handler, except here we should
    # insert into persistence - after doing whatever extra
    # validation we might require. We will assume that this is done,
    # and we now have a user instance with an assigned ID value:
    user = User(name=data.name, email=data.email, id=uuid4())

    await memory_store.set(data.email, str(user.id))
    MOCK_DB[str(user.id)] = user
    # we are creating a session the same as we do in the
    # 'login_handler' above:
    request.set_session({"user_id": str(user.id)})

    # and again, you can add whatever logic is required here, we
    # will simply return the user:
    return user


# the endpoint below requires the user to be already authenticated
# to be able to access it.
@get("/user", sync_to_thread=False)
def get_user(request: Request[User, Dict[Literal["user_id"], str], Any]) -> Any:
    # because this route requires authentication, we can access
    # `request.user`, which is the authenticated user returned
    # by the 'retrieve_user_handler' function we passed to SessionAuth.
    return request.user


# We add the session security schema to the OpenAPI config.
openapi_config = OpenAPIConfig(
    title="My API",
    version="1.0.0",
)

session_auth = SessionAuth[User, ServerSideSessionBackend](
    retrieve_user_handler=retrieve_user_handler,
    # we must pass a config for a session backend.
    # all session backends are supported
    session_backend_config=ServerSideSessionConfig(),
    # exclude any URLs that should not have authentication.
    # We exclude the documentation URLs, signup and login.
    exclude=["/login", "/signup", "/schema"],
)


# We initialize the app instance, passing to it the 'session_auth.on_app_init' and the 'openapi_config'.
app = Litestar(
    route_handlers=[login, signup, get_user],
    on_app_init=[session_auth.on_app_init],
    openapi_config=openapi_config,
)



from typing import Iterator

import pytest

from litestar import Litestar, MediaType, get
from litestar.status_codes import HTTP_200_OK
from litestar.testing import TestClient


@get(path="/health-check", media_type=MediaType.TEXT, sync_to_thread=False)
def health_check() -> str:
    return "healthy"


app = Litestar(route_handlers=[health_check], debug=True)


@pytest.fixture(scope="function")
def test_client() -> Iterator[TestClient[Litestar]]:
    with TestClient(app=app) as client:
        yield client


def test_health_check_with_fixture(test_client: TestClient[Litestar]) -> None:
    response = test_client.get("/health-check")
    assert response.status_code == HTTP_200_OK
    assert response.text == "healthy"



from __future__ import annotations

from pathlib import Path

try:
    import rich_click as click
except ImportError:
    import click  # type: ignore[no-redef]

from click import Path as ClickPath

from ._utils import LitestarEnv, LitestarExtensionGroup
from .commands import core, schema, sessions

__all__ = ("litestar_group",)


@click.group(cls=LitestarExtensionGroup, context_settings={"help_option_names": ["-h", "--help"]})
@click.option("--app", "app_path", help="Module path to a Litestar application", is_eager=True)
@click.option(
    "--app-dir",
    help="Look for APP in the specified directory, by adding this to the PYTHONPATH. Defaults to the current working directory.",
    default=None,
    type=ClickPath(dir_okay=True, file_okay=False, path_type=Path),
    show_default=False,
    is_eager=True,
)
@click.pass_context
def litestar_group(ctx: click.Context, app_path: str | None, app_dir: Path | None = None) -> None:
    """Litestar CLI.

    The application to will be automatically discovered if it's in one of these
    canonical paths: 'app.py', 'asgi.py', 'application.py' or 'app/__init__.py'.
    When auto-discovering application factories, functions with the name 'create_app'
    are considered, or functions that are annotated as returning a 'Litestar'
    instance.

    Alternatively, the application can be specified explicitly via the '--app' option
    ('litestar --app=<module name>.<submodule>:<app instance or factory>') or the
    'LITESTAR_APP' environment variable of the same name.
    """
    if ctx.obj is None:  # env has not been loaded yet, so we can lazy load it
        ctx.obj = lambda: LitestarEnv.from_env(app_path, app_dir=app_dir)


# add sub commands here

litestar_group.add_command(core.info_command)  # pyright: ignore
litestar_group.add_command(core.run_command)  # pyright: ignore
litestar_group.add_command(core.routes_command)  # pyright: ignore
litestar_group.add_command(core.version_command)  # pyright: ignore
litestar_group.add_command(sessions.sessions_group)  # pyright: ignore
litestar_group.add_command(schema.schema_group)  # pyright: ignore



from typing import AsyncGenerator, List, Optional

from sqlalchemy import select
from sqlalchemy.exc import IntegrityError, NoResultFound
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column

from litestar import Litestar, get, post, put
from litestar.exceptions import ClientException, NotFoundException
from litestar.plugins.sqlalchemy import SQLAlchemyAsyncConfig, SQLAlchemyPlugin
from litestar.status_codes import HTTP_409_CONFLICT


class Base(DeclarativeBase): ...


class TodoItem(Base):
    __tablename__ = "todo_items"

    title: Mapped[str] = mapped_column(primary_key=True)
    done: Mapped[bool]


async def provide_transaction(db_session: AsyncSession) -> AsyncGenerator[AsyncSession, None]:
    try:
        async with db_session.begin():
            yield db_session
    except IntegrityError as exc:
        raise ClientException(
            status_code=HTTP_409_CONFLICT,
            detail=str(exc),
        ) from exc


async def get_todo_by_title(todo_name: str, session: AsyncSession) -> TodoItem:
    query = select(TodoItem).where(TodoItem.title == todo_name)
    result = await session.execute(query)
    try:
        return result.scalar_one()
    except NoResultFound as e:
        raise NotFoundException(detail=f"TODO {todo_name!r} not found") from e


async def get_todo_list(done: Optional[bool], session: AsyncSession) -> List[TodoItem]:
    query = select(TodoItem)
    if done is not None:
        query = query.where(TodoItem.done.is_(done))

    result = await session.execute(query)
    return list(result.scalars().all())


@get("/")
async def get_list(transaction: AsyncSession, done: Optional[bool] = None) -> List[TodoItem]:
    return await get_todo_list(done, transaction)


@post("/")
async def add_item(data: TodoItem, transaction: AsyncSession) -> TodoItem:
    transaction.add(data)
    return data


@put("/{item_title:str}")
async def update_item(item_title: str, data: TodoItem, transaction: AsyncSession) -> TodoItem:
    todo_item = await get_todo_by_title(item_title, transaction)
    todo_item.title = data.title
    todo_item.done = data.done
    return todo_item


db_config = SQLAlchemyAsyncConfig(
    connection_string="sqlite+aiosqlite:///todo.sqlite",
    metadata=Base.metadata,
    create_all=True,
    before_send_handler="autocommit",
)

app = Litestar(
    [get_list, add_item, update_item],
    dependencies={"transaction": provide_transaction},
    plugins=[SQLAlchemyPlugin(db_config)],
)



# ruff: noqa: F401
# pyright: reportUnusedImport=false
from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.utils import warn_deprecation

__all__ = (
    "GUID",
    "ORA_JSONB",
    "BigIntIdentity",
    "DateTimeUTC",
    "JsonB",
)


def __getattr__(attr_name: str) -> object:
    if attr_name in __all__:
        from advanced_alchemy.types import (
            GUID,
            ORA_JSONB,
            BigIntIdentity,
            DateTimeUTC,
            JsonB,
        )

        warn_deprecation(
            deprecated_name=f"litestar.contrib.sqlalchemy.{attr_name}",
            version="2.12",
            kind="import",
            removal_in="3.0",
            info=f"importing {attr_name} from 'litestar.contrib.sqlalchemy' is deprecated, please "
            f"import it from 'advanced_alchemy.extensions.litestar.types' instead",
        )
        value = globals()[attr_name] = locals()[attr_name]
        return value

    raise AttributeError(f"module {__name__!r} has no attribute {attr_name!r}")  # pragma: no cover


if TYPE_CHECKING:
    from advanced_alchemy.types import (
        GUID,
        ORA_JSONB,
        BigIntIdentity,
        DateTimeUTC,
        JsonB,
    )



import time

from litestar import Controller, Litestar, get
from litestar.datastructures import CacheControlHeader


class MyController(Controller):
    cache_control = CacheControlHeader(max_age=86_400, public=True)

    @get("/chance_of_rain", sync_to_thread=False)
    def get_chance_of_rain(self) -> float:
        """This endpoint uses the cache control value defined in the controller which overrides the app value."""
        return 0.5

    @get("/timestamp", cache_control=CacheControlHeader(no_store=True), sync_to_thread=False)
    def get_server_time(self) -> float:
        """This endpoint overrides the cache control value defined in the controller."""
        return time.time()


@get("/population", sync_to_thread=False)
def get_population_count() -> int:
    """This endpoint will use the cache control defined in the app."""
    return 100000


app = Litestar(
    route_handlers=[MyController, get_population_count],
    cache_control=CacheControlHeader(max_age=2_628_288, public=True),
)



from litestar.template.base import TemplateEngineProtocol, TemplateProtocol
from litestar.template.config import TemplateConfig

__all__ = ("TemplateConfig", "TemplateEngineProtocol", "TemplateProtocol")



import time
from typing import Dict

from litestar import Litestar, WebSocket, get, websocket
from litestar.datastructures import MutableScopeHeaders
from litestar.enums import ScopeType
from litestar.middleware import ASGIMiddleware
from litestar.types import ASGIApp, Message, Receive, Scope, Send


class MyMiddleware(ASGIMiddleware):
    scopes = (ScopeType.HTTP,)
    exclude_path_pattern = ("first_path", "second_path")
    exclude_opt_key = "exclude_from_my_middleware"

    async def handle(self, scope: Scope, receive: Receive, send: Send, next_app: ASGIApp) -> None:
        start_time = time.monotonic()

        async def send_wrapper(message: "Message") -> None:
            if message["type"] == "http.response.start":
                process_time = time.monotonic() - start_time
                headers = MutableScopeHeaders.from_message(message=message)
                headers["X-Process-Time"] = str(process_time)
            await send(message)

        await next_app(scope, receive, send_wrapper)


@websocket("/my-websocket")
async def websocket_handler(socket: WebSocket) -> None:
    """
    Websocket handler - is excluded because the middleware scopes includes 'ScopeType.HTTP'
    """
    await socket.accept()
    await socket.send_json({"hello": "websocket"})
    await socket.close()


@get("/first_path", sync_to_thread=False)
def first_handler() -> Dict[str, str]:
    """Handler is excluded due to regex pattern matching "first_path"."""
    return {"hello": "first"}


@get("/second_path", sync_to_thread=False)
def second_handler() -> Dict[str, str]:
    """Handler is excluded due to regex pattern matching "second_path"."""
    return {"hello": "second"}


@get("/third_path", exclude_from_my_middleware=True, sync_to_thread=False)
def third_handler() -> Dict[str, str]:
    """Handler is excluded due to the opt key 'exclude_from_my_middleware' matching the middleware 'exclude_opt_key'."""
    return {"hello": "third"}


@get("/greet", sync_to_thread=False)
def not_excluded_handler() -> Dict[str, str]:
    """This handler is not excluded, and thus the middleware will execute on every incoming request to it."""
    return {"hello": "world"}


app = Litestar(
    route_handlers=[
        websocket_handler,
        first_handler,
        second_handler,
        third_handler,
        not_excluded_handler,
    ],
    middleware=[MyMiddleware()],
)



from pathlib import Path
from types import ModuleType
from typing import Any, Callable, Dict, List

import pytest
from typing_extensions import Annotated

from litestar import HttpMethod, Litestar, WebSocket, delete, get, patch, post, put, route
from litestar.exceptions import ImproperlyConfiguredException, ValidationException
from litestar.handlers.http_handlers import HTTPRouteHandler
from litestar.params import Body
from litestar.response import File, Redirect
from litestar.status_codes import (
    HTTP_100_CONTINUE,
    HTTP_200_OK,
    HTTP_304_NOT_MODIFIED,
    HTTP_307_TEMPORARY_REDIRECT,
)
from tests.models import DataclassPerson


def test_route_handler_validation_http_method() -> None:
    # doesn't raise for http methods
    for value in (*list(HttpMethod), *[x.upper() for x in list(HttpMethod)]):
        assert route(http_method=value)  # type: ignore[arg-type, truthy-bool]

    # raises for invalid values
    with pytest.raises(ValidationException):
        HTTPRouteHandler(http_method="deleze")  # type: ignore[arg-type]

    # also when passing an empty list
    with pytest.raises(ImproperlyConfiguredException):
        route(http_method=[], status_code=HTTP_200_OK)

    # also when passing malformed tokens
    with pytest.raises(ValidationException):
        route(http_method=[HttpMethod.GET, "poft"], status_code=HTTP_200_OK)  # type: ignore[list-item]


async def test_function_validation() -> None:
    with pytest.raises(ImproperlyConfiguredException):

        @get(path="/")
        def method_with_no_annotation():  # type: ignore[no-untyped-def]
            pass

        Litestar(route_handlers=[method_with_no_annotation])

        method_with_no_annotation.on_registration(Litestar())

    with pytest.raises(ImproperlyConfiguredException):

        @delete(path="/")
        def method_with_no_content() -> Dict[str, str]:
            return {}

        Litestar(route_handlers=[method_with_no_content])

        method_with_no_content.on_registration(Litestar())

    with pytest.raises(ImproperlyConfiguredException):

        @get(path="/", status_code=HTTP_304_NOT_MODIFIED)
        def method_with_not_modified() -> Dict[str, str]:
            return {}

        Litestar(route_handlers=[method_with_not_modified])

        method_with_not_modified.on_registration(Litestar())

    with pytest.raises(ImproperlyConfiguredException):

        @get(path="/", status_code=HTTP_100_CONTINUE)
        def method_with_status_lower_than_200() -> Dict[str, str]:
            return {}

        Litestar(route_handlers=[method_with_status_lower_than_200])

        method_with_status_lower_than_200.on_registration(Litestar())

    @get(path="/", status_code=HTTP_307_TEMPORARY_REDIRECT)
    def redirect_method() -> Redirect:
        return Redirect("/test")

    Litestar(route_handlers=[redirect_method])

    redirect_method.on_registration(Litestar())

    @get(path="/")
    def file_method() -> File:
        return File(path=Path("."), filename="test_validations.py")

    Litestar(route_handlers=[file_method])

    file_method.on_registration(Litestar())

    assert not file_method.media_type

    with pytest.raises(ImproperlyConfiguredException):

        @get(path="/test")
        def test_function_1(socket: WebSocket) -> None:
            return None

        test_function_1.on_registration(Litestar())

    with pytest.raises(ImproperlyConfiguredException):

        @get("/person")
        def test_function_2(self, data: DataclassPerson) -> None:  # type: ignore[no-untyped-def]
            return None

        Litestar(route_handlers=[test_function_2])

        test_function_2.on_registration(Litestar())


@pytest.mark.parametrize(
    ("return_annotation", "should_raise"),
    [
        ("None", False),
        ("Response[None]", False),
        ("int", True),
        ("Response[int]", True),
        ("Response", True),
    ],
)
def test_204_response_annotations(
    return_annotation: str, should_raise: bool, create_module: Callable[[str], ModuleType]
) -> None:
    module = create_module(
        f"""
from litestar import get
from litestar.response import Response
from litestar.status_codes import HTTP_204_NO_CONTENT

@get(path="/", status_code=HTTP_204_NO_CONTENT)
def no_response_handler() -> {return_annotation}:
    pass
"""
    )

    if should_raise:
        with pytest.raises(ImproperlyConfiguredException):
            Litestar(route_handlers=[module.no_response_handler])
        return

    Litestar(route_handlers=[module.no_response_handler])


@pytest.mark.parametrize("decorator", [post, put, patch])
def test_body_param_with_non_bytes_annotation_raises(decorator: Callable[..., Any]) -> None:
    def handler_fn(body: List[str]) -> None:
        pass

    with pytest.raises(ImproperlyConfiguredException, match="Invalid type annotation for 'body' parameter"):
        Litestar([decorator()(handler_fn)])


@pytest.mark.parametrize("decorator", [post, put, patch])
def test_body_param_with_metadata_allowed(decorator: Callable[..., Any]) -> None:
    def handler_fn(body: Annotated[bytes, Body(title="something")]) -> None:
        pass

    # we expect no error here, even though the type isn't directly 'bytes' but has
    # metadata attached to it
    Litestar([decorator()(handler_fn)])



from __future__ import annotations

from dataclasses import dataclass

from litestar import Litestar, post
from litestar.dto import DataclassDTO, DTOConfig


@dataclass
class Person:
    name: str
    age: int
    email: str
    id: int


class ReadDTO(DataclassDTO[Person]):
    config = DTOConfig(exclude={"email"})


class WriteDTO(DataclassDTO[Person]):
    config = DTOConfig(exclude={"id"})


@post("/person", dto=WriteDTO, return_dto=ReadDTO, sync_to_thread=False)
def create_person(data: Person) -> Person:
    # Logic for persisting the person goes here
    return data


app = Litestar(route_handlers=[create_person])



from http import HTTPStatus

from litestar import Litestar, get
from litestar.config.cors import CORSConfig
from litestar.testing import TestClient


@get("/endpoint")
async def handler() -> str:
    return "Hello, world!"


cors_config = CORSConfig(
    allow_methods=["GET"], allow_origins=["https://allowed-origin.com", "https://another-allowed-origin.com"]
)
app = Litestar(route_handlers=[handler], cors_config=cors_config)


def test_cors_with_allowed_origins() -> None:
    with TestClient(app) as client:
        response = client.options(
            "/custom-options", headers={"Origin": "https://allowed-origin.com", "Access-Control-Request-Method": "GET"}
        )
        assert response.status_code == HTTPStatus.NO_CONTENT
        assert response.headers["access-control-allow-origin"] == "https://allowed-origin.com"

        response = client.options(
            "/custom-options",
            headers={"Origin": "https://another-allowed-origin.com", "Access-Control-Request-Method": "GET"},
        )
        assert response.status_code == HTTPStatus.NO_CONTENT
        assert response.headers["access-control-allow-origin"] == "https://another-allowed-origin.com"


def test_cors_with_disallowed_origin() -> None:
    with TestClient(app) as client:
        response = client.options(
            "/custom-options",
            headers={"Origin": "https://disallowed-origin.com", "Access-Control-Request-Method": "GET"},
        )
        assert response.status_code == HTTPStatus.BAD_REQUEST
        assert "access-control-allow-origin" not in response.headers



from __future__ import annotations

from abc import ABC
from typing import TYPE_CHECKING, Any, Generator, Generic, Iterable, Mapping, TypeVar

from multidict import MultiDict as BaseMultiDict
from multidict import MultiDictProxy, MultiMapping

from litestar.datastructures.upload_file import UploadFile

if TYPE_CHECKING:
    from typing_extensions import Self


__all__ = ("FormMultiDict", "ImmutableMultiDict", "MultiDict", "MultiMixin")


T = TypeVar("T")


class MultiMixin(Generic[T], MultiMapping[T], ABC):
    """Mixin providing common methods for multi dicts, used by :class:`ImmutableMultiDict` and :class:`MultiDict`"""

    def dict(self) -> dict[str, list[Any]]:
        """Return the multi-dict as a dict of lists.

        Returns:
            A dict of lists
        """
        return {k: self.getall(k) for k in set(self.keys())}

    def multi_items(self) -> Generator[tuple[str, T], None, None]:
        """Get all keys and values, including duplicates.

        Returns:
            A list of tuples containing key-value pairs
        """
        for key in set(self):
            for value in self.getall(key):
                yield key, value


class MultiDict(BaseMultiDict[T], MultiMixin[T], Generic[T]):
    """MultiDict, using :class:`MultiDict <multidict.MultiDictProxy>`."""

    def __init__(self, args: MultiMapping | Mapping[str, T] | Iterable[tuple[str, T]] | None = None) -> None:
        """Initialize ``MultiDict`` from a`MultiMapping``,
        :class:`Mapping <typing.Mapping>` or an iterable of tuples.

        Args:
            args: Mapping-like structure to create the ``MultiDict`` from
        """
        super().__init__(args or {})

    def immutable(self) -> ImmutableMultiDict[T]:
        """Create an.

        :class:`ImmutableMultiDict` view.

        Returns:
            An immutable multi dict
        """
        return ImmutableMultiDict[T](self)  # pyright: ignore

    def copy(self) -> Self:
        """Return a shallow copy"""
        return type(self)(list(self.multi_items()))


class ImmutableMultiDict(MultiDictProxy[T], MultiMixin[T], Generic[T]):
    """Immutable MultiDict, using class:`MultiDictProxy <multidict.MultiDictProxy>`."""

    def __init__(self, args: MultiMapping | Mapping[str, Any] | Iterable[tuple[str, Any]] | None = None) -> None:
        """Initialize ``ImmutableMultiDict`` from a `MultiMapping``,
        :class:`Mapping <typing.Mapping>` or an iterable of tuples.

        Args:
            args: Mapping-like structure to create the ``ImmutableMultiDict`` from
        """
        super().__init__(BaseMultiDict(args or {}))

    def mutable_copy(self) -> MultiDict[T]:
        """Create a mutable copy as a :class:`MultiDict`

        Returns:
            A mutable multi dict
        """
        return MultiDict(list(self.multi_items()))

    def copy(self) -> Self:  # type: ignore[override]
        """Return a shallow copy"""
        return type(self)(self.items())


class FormMultiDict(ImmutableMultiDict[Any]):
    """MultiDict for form data."""

    @classmethod
    def from_form_data(cls, form_data: dict[str, list[str] | str | UploadFile]) -> FormMultiDict:
        """Create a FormMultiDict from form data.

        Args:
            form_data: Form data to create the FormMultiDict from.

        Returns:
            A FormMultiDict instance
        """
        # Convert form_data to a list[tuple[str, str | UploadFile]] before passing it
        # to FormMultiDict so multi-keys can be accessed properly
        items = []
        for k, v in form_data.items():
            if not isinstance(v, list):
                items.append((k, v))
            else:
                for sv in v:
                    items.append((k, sv))
        return cls(items)

    async def close(self) -> None:
        """Close all files in the multi-dict.

        Returns:
            None
        """
        for _, value in self.multi_items():
            if isinstance(value, UploadFile):
                await value.close()



from typing import Any, Dict, List, Tuple

from typing_extensions import Annotated

from litestar import Litestar, post
from litestar.datastructures import UploadFile
from litestar.enums import RequestEncodingType
from litestar.params import Body


@post(path="/")
async def handle_file_upload(
    data: Annotated[List[UploadFile], Body(media_type=RequestEncodingType.MULTI_PART)],
) -> Dict[str, Tuple[str, str, Any]]:
    result = {}

    for file in data:
        content = await file.read()
        result[file.filename] = (len(content), file.content_type, file.headers)

    return result


app = Litestar(route_handlers=[handle_file_upload])



from __future__ import annotations

import dataclasses
from dataclasses import replace
from typing import TYPE_CHECKING, Generic, TypeVar

import msgspec.inspect
from msgspec import NODEFAULT, Struct, structs

from litestar.dto.base_dto import AbstractDTO
from litestar.dto.data_structures import DTOFieldDefinition
from litestar.dto.field import DTO_FIELD_META_KEY, DTOField, extract_dto_field
from litestar.plugins.core._msgspec import kwarg_definition_from_field
from litestar.types.empty import Empty

if TYPE_CHECKING:
    from typing import Any, Collection, Generator

    from litestar.typing import FieldDefinition


__all__ = ("MsgspecDTO",)

T = TypeVar("T", bound="Struct | Collection[Struct]")


def _default_or_empty(value: Any) -> Any:
    return Empty if value is NODEFAULT else value


def _default_or_none(value: Any) -> Any:
    return None if value is NODEFAULT else value


class MsgspecDTO(AbstractDTO[T], Generic[T]):
    """Support for domain modelling with Msgspec."""

    @classmethod
    def generate_field_definitions(cls, model_type: type[Struct]) -> Generator[DTOFieldDefinition, None, None]:
        msgspec_fields = {f.name: f for f in structs.fields(model_type)}

        inspect_fields: dict[str, msgspec.inspect.Field] = {
            field.name: field
            for field in msgspec.inspect.type_info(model_type).fields  # type: ignore[attr-defined]
        }

        property_fields = cls.get_property_fields(model_type)

        for key, field_definition in cls.get_model_type_hints(model_type).items():
            kwarg_definition, extra = kwarg_definition_from_field(inspect_fields[key])
            field_definition = dataclasses.replace(field_definition, kwarg_definition=kwarg_definition)
            field_definition.extra.update(extra)
            dto_field = extract_dto_field(field_definition, field_definition.extra)
            field_definition.extra.pop(DTO_FIELD_META_KEY, None)
            msgspec_field = msgspec_fields[key]

            yield replace(
                DTOFieldDefinition.from_field_definition(
                    field_definition=field_definition,
                    dto_field=dto_field,
                    model_name=model_type.__name__,
                    default_factory=_default_or_none(msgspec_field.default_factory),
                ),
                default=_default_or_empty(msgspec_field.default),
                name=key,
            )

        for key, property_field in property_fields.items():
            if key.startswith("_"):
                continue

            yield DTOFieldDefinition.from_field_definition(
                property_field,
                model_name=model_type.__name__,
                default_factory=None,
                dto_field=DTOField(mark="read-only"),
            )

    @classmethod
    def detect_nested_field(cls, field_definition: FieldDefinition) -> bool:
        return field_definition.is_subclass_of(Struct)



# ruff: noqa: UP006
from __future__ import annotations

import inspect
import logging
from collections.abc import AsyncGenerator
from contextlib import asynccontextmanager
from dataclasses import fields
from typing import TYPE_CHECKING, Any, Callable, List, Tuple
from unittest.mock import MagicMock, Mock, PropertyMock

import pytest
from click import Group
from pytest import MonkeyPatch

from litestar import Litestar, MediaType, Request, Response, get
from litestar._asgi.asgi_router import ASGIRouter
from litestar.config.app import AppConfig, ExperimentalFeatures
from litestar.config.response_cache import ResponseCacheConfig
from litestar.contrib.sqlalchemy.plugins import SQLAlchemySerializationPlugin
from litestar.datastructures import MutableScopeHeaders, State
from litestar.events.emitter import SimpleEventEmitter
from litestar.exceptions import (
    ImproperlyConfiguredException,
    InternalServerException,
    LitestarWarning,
    NotFoundException,
)
from litestar.logging.config import LoggingConfig
from litestar.plugins import CLIPluginProtocol
from litestar.router import Router
from litestar.status_codes import HTTP_200_OK, HTTP_400_BAD_REQUEST, HTTP_500_INTERNAL_SERVER_ERROR
from litestar.testing import TestClient, create_test_client

if TYPE_CHECKING:
    from typing import Dict

    from litestar.types import Message, Scope


@pytest.fixture()
def app_config_object() -> AppConfig:
    return AppConfig(
        after_exception=[],
        after_request=None,
        after_response=None,
        allowed_hosts=[],
        before_request=None,
        before_send=[],
        response_cache_config=ResponseCacheConfig(),
        cache_control=None,
        compression_config=None,
        cors_config=None,
        csrf_config=None,
        debug=False,
        dependencies={},
        etag=None,
        event_emitter_backend=SimpleEventEmitter,
        exception_handlers={},
        guards=[],
        listeners=[],
        logging_config=None,
        middleware=[],
        multipart_form_part_limit=1000,
        on_shutdown=[],
        on_startup=[],
        openapi_config=None,
        opt={},
        parameters={},
        plugins=[],
        request_class=None,
        response_class=None,
        response_cookies=[],
        response_headers=[],
        route_handlers=[],
        security=[],
        static_files_config=[],
        tags=[],
        template_config=None,
        websocket_class=None,
    )


def test_access_openapi_schema_raises_if_not_configured() -> None:
    """Test that accessing the openapi schema raises if not configured."""
    app = Litestar(openapi_config=None)
    with pytest.raises(ImproperlyConfiguredException):
        app.openapi_schema


def test_set_debug_updates_logging_level() -> None:
    app = Litestar()

    assert app.logger is not None
    assert app.logger.level == logging.INFO  # type: ignore[attr-defined]

    app.debug = True
    assert app.logger.level == logging.DEBUG  # type: ignore[attr-defined]

    app.debug = False
    assert app.logger.level == logging.INFO  # type: ignore[attr-defined]


@pytest.mark.parametrize("env_name,app_attr", [("LITESTAR_DEBUG", "debug"), ("LITESTAR_PDB", "pdb_on_exception")])
@pytest.mark.parametrize(
    "env_value,app_value,expected_value",
    [
        (None, None, False),
        (None, False, False),
        (None, True, True),
        ("0", None, False),
        ("0", False, False),
        ("0", True, True),
        ("1", None, True),
        ("1", False, False),
        ("1", True, True),
    ],
)
@pytest.mark.filterwarnings("ignore::litestar.utils.warnings.LitestarWarning:")
def test_set_env_flags(
    monkeypatch: MonkeyPatch,
    env_value: str | None,
    app_value: bool | None,
    expected_value: bool,
    env_name: str,
    app_attr: str,
) -> None:
    if env_value is not None:
        monkeypatch.setenv(env_name, env_value)
    else:
        monkeypatch.delenv(env_name, raising=False)

    app = Litestar(**{app_attr: app_value})  # type: ignore[arg-type]

    assert getattr(app, app_attr) is expected_value


def test_warn_pdb_on_exception() -> None:
    with pytest.warns(LitestarWarning, match="Debugger"):
        Litestar(pdb_on_exception=True)


def test_app_params_defined_on_app_config_object() -> None:
    """Ensures that all parameters to the `Litestar` constructor are present on the `AppConfig` object."""
    litestar_signature = inspect.signature(Litestar)
    app_config_fields = {f.name for f in fields(AppConfig)}
    for name in litestar_signature.parameters:
        if name in {"on_app_init", "initial_state", "_preferred_validation_backend"}:
            continue
        assert name in app_config_fields
    # ensure there are not fields defined on AppConfig that aren't in the Litestar signature
    assert not (app_config_fields - set(litestar_signature.parameters.keys()))


def test_app_config_object_used(app_config_object: AppConfig, monkeypatch: pytest.MonkeyPatch) -> None:
    """Ensure that the properties on the `AppConfig` object are accessed within the `Litestar` constructor.

    In the test we replace every field on the `AppConfig` type with a property mock so that we can check that it has at
    least been accessed. It doesn't actually check that we do the right thing with it, but is a guard against the case
    of adding a parameter to the `Litestar` signature and to the `AppConfig` object, and using the value from the
    parameter downstream from construction of the `AppConfig` object.
    """

    # replace each field on the `AppConfig` object with a `PropertyMock`, this allows us to assert that the properties
    # have been accessed during app instantiation.
    property_mocks: List[Tuple[str, Mock]] = []
    for field in fields(AppConfig):
        property_mock = PropertyMock()
        property_mocks.append((field.name, property_mock))
        monkeypatch.setattr(type(app_config_object), field.name, property_mock, raising=False)

    # Things that we don't actually need to call for this test
    monkeypatch.setattr(Litestar, "register", MagicMock())
    monkeypatch.setattr(Litestar, "_create_asgi_handler", MagicMock())
    monkeypatch.setattr(Router, "__init__", MagicMock())
    monkeypatch.setattr(ASGIRouter, "__init__", MagicMock(return_value=None))

    # instantiates the app with an `on_app_config` that returns our patched `AppConfig` object.
    Litestar(on_app_init=[MagicMock(return_value=app_config_object)])

    # this ensures that each of the properties of the `AppConfig` object have been accessed within `Litestar.__init__()`
    for name, mock in property_mocks:
        assert mock.called, f"expected {name} to be called"


def test_app_debug_create_logger() -> None:
    app = Litestar([], debug=True)

    assert app.logging_config
    assert app.logging_config.loggers["litestar"]["level"] == "DEBUG"  # type: ignore[attr-defined]


def test_app_debug_explicitly_disable_logging() -> None:
    app = Litestar([], logging_config=None)

    assert not app.logging_config


def test_app_debug_update_logging_config() -> None:
    logging_config = LoggingConfig()
    app = Litestar([], logging_config=logging_config, debug=True)

    assert app.logging_config is logging_config
    assert app.logging_config.loggers["litestar"]["level"] == "DEBUG"  # type: ignore[attr-defined]


def test_set_state() -> None:
    def modify_state_in_hook(app_config: AppConfig) -> AppConfig:
        assert isinstance(app_config.state, State)
        app_config.state["c"] = "D"
        app_config.state["e"] = "f"
        return app_config

    app = Litestar(state=State({"a": "b", "c": "d"}), on_app_init=[modify_state_in_hook])
    assert app.state._state == {"a": "b", "c": "D", "e": "f"}


async def test_dont_override_initial_state(create_scope: Callable[..., Scope]) -> None:
    app = Litestar()

    scope = create_scope(headers=[], state={"foo": "bar"})

    async def send(message: Message) -> None:
        pass

    async def receive() -> None:
        pass

    await app(scope, receive, send)  # type: ignore[arg-type]

    assert scope["state"].get("foo") == "bar"


def test_app_from_config(app_config_object: AppConfig) -> None:
    Litestar.from_config(app_config_object)


def test_before_send() -> None:
    @get("/test")
    def handler() -> Dict[str, str]:
        return {"key": "value"}

    async def before_send_hook_handler(message: Message, scope: Scope) -> None:
        if message["type"] == "http.response.start":
            headers = MutableScopeHeaders(message)
            headers.add("My-Header", Litestar.from_scope(scope).state.message)

    def on_startup(app: Litestar) -> None:
        app.state.message = "value injected during send"

    with create_test_client(handler, on_startup=[on_startup], before_send=[before_send_hook_handler]) as client:
        response = client.get("/test")
        assert response.status_code == HTTP_200_OK
        assert response.headers.get("My-Header") == "value injected during send"


def test_using_custom_http_exception_handler() -> None:
    @get("/{param:int}")
    def my_route_handler(param: int) -> None: ...

    def my_custom_handler(_: Request[Any, Any, State], __: Exception) -> Response[str]:
        return Response(content="custom message", media_type=MediaType.TEXT, status_code=HTTP_400_BAD_REQUEST)

    with create_test_client(my_route_handler, exception_handlers={NotFoundException: my_custom_handler}) as client:
        response = client.get("/abc")
        assert response.text == "custom message"
        assert response.status_code == HTTP_400_BAD_REQUEST


def test_debug_response_created() -> None:
    # this will test exception causes are recorded in output
    # since frames include code in context we should not raise
    # exception directly
    def exception_thrower() -> float:
        return 1 / 0

    @get("/")
    def my_route_handler() -> None:
        try:
            exception_thrower()
        except Exception as e:
            raise InternalServerException() from e

    app = Litestar(route_handlers=[my_route_handler], debug=True)
    client = TestClient(app=app)

    response = client.get("/")
    assert response.status_code == HTTP_500_INTERNAL_SERVER_ERROR
    assert "text/plain" in response.headers["content-type"]

    response = client.get("/", headers={"accept": "text/html"})
    assert response.status_code == HTTP_500_INTERNAL_SERVER_ERROR
    assert "text/html" in response.headers["content-type"]
    assert "ZeroDivisionError" in response.text


def test_handler_error_return_status_500() -> None:
    @get("/")
    def my_route_handler() -> None:
        raise KeyError("custom message")

    with create_test_client(my_route_handler) as client:
        response = client.get("/")
        assert response.status_code == HTTP_500_INTERNAL_SERVER_ERROR


def test_lifespan() -> None:
    events: list[str] = []
    counter = {"value": 0}

    def sync_function_without_app() -> None:
        events.append("sync_function_without_app")
        counter["value"] += 1

    async def async_function_without_app() -> None:
        events.append("async_function_without_app")
        counter["value"] += 1

    def sync_function_with_app(app: Litestar) -> None:
        events.append("sync_function_with_app")
        assert app is not None
        assert isinstance(app.state, State)
        counter["value"] += 1
        app.state.x = True

    async def async_function_with_app(app: Litestar) -> None:
        events.append("async_function_with_app")
        assert app is not None
        assert isinstance(app.state, State)
        counter["value"] += 1
        app.state.y = True

    with create_test_client(
        [],
        on_startup=[
            sync_function_without_app,
            async_function_without_app,
            sync_function_with_app,
            async_function_with_app,
        ],
        on_shutdown=[
            sync_function_without_app,
            async_function_without_app,
            sync_function_with_app,
            async_function_with_app,
        ],
    ) as client:
        assert counter["value"] == 4
        assert client.app.state.x
        assert client.app.state.y
        counter["value"] = 0
        assert counter["value"] == 0
    assert counter["value"] == 4
    assert events == [
        "sync_function_without_app",
        "async_function_without_app",
        "sync_function_with_app",
        "async_function_with_app",
        "sync_function_without_app",
        "async_function_without_app",
        "sync_function_with_app",
        "async_function_with_app",
    ]


def test_registering_route_handler_generates_openapi_docs() -> None:
    def fn() -> None:
        return

    app = Litestar(route_handlers=[])
    assert app.openapi_schema

    app.register(get("/path1")(fn))

    assert app.openapi_schema.paths is not None
    assert app.openapi_schema.paths.get("/path1")

    app.register(get("/path2")(fn))
    assert app.openapi_schema.paths.get("/path1")
    assert app.openapi_schema.paths.get("/path2")


def test_plugin_properties() -> None:
    class FooPlugin(CLIPluginProtocol):
        def on_cli_init(self, cli: Group) -> None:
            return

    app = Litestar(plugins=[FooPlugin(), SQLAlchemySerializationPlugin()])

    assert app.openapi_schema_plugins == list(app.plugins.openapi)
    assert app.cli_plugins == list(app.plugins.cli)
    assert app.serialization_plugins == list(app.plugins.serialization)


def test_plugin_registry() -> None:
    class FooPlugin(CLIPluginProtocol):
        def on_cli_init(self, cli: Group) -> None:
            return

    foo = FooPlugin()

    app = Litestar(plugins=[foo])

    assert foo in app.plugins.cli


def test_lifespan_context_and_shutdown_hook_execution_order() -> None:
    events: list[str] = []
    counter = {"value": 0}

    @asynccontextmanager
    async def lifespan_context_1(app: Litestar) -> AsyncGenerator[None, None]:
        try:
            yield
        finally:
            events.append("ctx_1")
            counter["value"] += 1

    @asynccontextmanager
    async def lifespan_context_2(app: Litestar) -> AsyncGenerator[None, None]:
        try:
            yield
        finally:
            events.append("ctx_2")
            counter["value"] += 1

    async def hook_a(app: Litestar) -> None:
        events.append("hook_a")
        counter["value"] += 1

    async def hook_b(app: Litestar) -> None:
        events.append("hook_b")
        counter["value"] += 1

    with create_test_client(
        route_handlers=[],
        lifespan=[
            lifespan_context_1,
            lifespan_context_2,
        ],
        on_shutdown=[hook_a, hook_b],
    ):
        assert counter["value"] == 0

    assert counter["value"] == 4
    assert events[0] == "ctx_2"
    assert events[1] == "ctx_1"
    assert events[2] == "hook_a"
    assert events[3] == "hook_b"


def test_use_dto_codegen_feature_flag_warns() -> None:
    with pytest.warns(LitestarWarning, match="Use of redundant experimental feature flag DTO_CODEGEN"):
        Litestar(experimental_features=[ExperimentalFeatures.DTO_CODEGEN])


def test_use_future_feature_flag_warns() -> None:
    app = Litestar(experimental_features=[ExperimentalFeatures.FUTURE])
    assert app.experimental_features == frozenset([ExperimentalFeatures.FUTURE])


def test_using_custom_path_parameter() -> None:
    @get()
    def my_route_handler() -> None: ...

    with create_test_client(my_route_handler, path="/abc") as client:
        response = client.get("/abc")
        assert response.status_code == HTTP_200_OK


def test_from_scope() -> None:
    mock = MagicMock()

    @get()
    def handler(scope: Scope) -> None:
        mock(Litestar.from_scope(scope))
        return

    app = Litestar(route_handlers=[handler])

    with TestClient(app) as client:
        client.get("/")

    mock.assert_called_once_with(app)



from piccolo.utils.pydantic import create_pydantic_model

from test_apps.piccolo_admin_test_app.home.tables import Task

# task models
TaskModelIn = create_pydantic_model(table=Task, model_name="TaskModelIn")
TaskModelOut = create_pydantic_model(
    table=Task,
    include_default_columns=True,
    model_name="TaskModelOut",
)



from __future__ import annotations

from typing import Any

from litestar.middleware._internal import cors
from litestar.utils.deprecation import warn_deprecation


def __getattr__(name: str) -> Any:
    if name == "CORSMiddleware":
        warn_deprecation(
            version="2.9",
            deprecated_name=name,
            kind="class",
            removal_in="3.0",
            info="CORS middleware has been removed from the public API.",
        )
        return cors.CORSMiddleware
    raise AttributeError(f"module {__name__} has no attribute {name}")



from litestar import Litestar, WebSocket, websocket
from litestar.channels import ChannelsPlugin
from litestar.channels.backends.memory import MemoryChannelsBackend


@websocket("/ws")
async def handler(socket: WebSocket, channels: ChannelsPlugin) -> None:
    await socket.accept()

    async with channels.start_subscription(["some_channel"]) as subscriber, subscriber.run_in_background(
        socket.send_text
    ):
        while True:
            response = await socket.receive_text()
            await socket.send_text(response)


app = Litestar(
    [handler],
    plugins=[ChannelsPlugin(backend=MemoryChannelsBackend(), channels=["some_channel"])],
)



from dataclasses import dataclass
from uuid import UUID, uuid4

from litestar import Litestar, post
from litestar.dto import DataclassDTO, DTOConfig, DTOData


@dataclass
class User:
    name: str
    email: str
    age: int
    id: UUID


class UserWriteDTO(DataclassDTO[User]):
    """Don't allow client to set the id."""

    config = DTOConfig(exclude={"id"})


@post("/users", dto=UserWriteDTO, return_dto=None, sync_to_thread=False)
def create_user(data: DTOData[User]) -> User:
    """Create an user."""
    return data.create_instance(id=uuid4())


app = Litestar(route_handlers=[create_user])

# run: /users -H "Content-Type: application/json" -d '{"name":"Peter", "email": "peter@example.com", "age":41}'



# ruff: noqa: F401
from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.utils import warn_deprecation

__all__ = ("PrometheusConfig",)


def __getattr__(attr_name: str) -> object:
    if attr_name in __all__:
        from litestar.plugins.prometheus import PrometheusConfig

        warn_deprecation(
            deprecated_name=f"litestar.contrib.prometheus.config.{attr_name}",
            version="2.13.0",
            kind="import",
            removal_in="3.0",
            info=f"importing {attr_name} from 'litestar.contrib.prometheus.config' is deprecated, please "
            f"import it from 'litestar.plugins.prometheus' instead",
        )
        value = globals()[attr_name] = locals()[attr_name]
        return value

    raise AttributeError(f"module {__name__!r} has no attribute {attr_name!r}")  # pragma: no cover


if TYPE_CHECKING:
    from litestar.plugins.prometheus import PrometheusConfig



from datetime import date, datetime, time, timedelta
from pathlib import Path
from typing import Type
from uuid import UUID

import pytest

from litestar import Litestar, Router, delete, get, patch, post, put
from litestar.exceptions import NoRouteMatchFoundException
from litestar.handlers import HTTPRouteHandler


@pytest.mark.parametrize("decorator", [get, post, patch, put, delete])
def test_route_reverse(decorator: Type[HTTPRouteHandler]) -> None:
    @decorator("/path-one/{param:str}", name="handler-name")  # type: ignore[call-arg]
    def handler() -> None:
        return None

    @decorator("/path-nameless")  # type: ignore[call-arg]
    def handler_nameless() -> None:
        return None

    @decorator("/path-two", name="handler-no-params")  # type: ignore[call-arg]
    def handler_no_params() -> None:
        return None

    @decorator("/multiple/{str_param:str}/params/{int_param:int}/", name="multiple-params-handler-name")  # type: ignore[call-arg]
    def handler2() -> None:
        return None

    @decorator(
        ["/handler3", "/handler3/{str_param:str}/", "/handler3/{str_param:str}/{int_param:int}/"],
        name="multiple-default-params",
    )  # type: ignore[call-arg]
    def handler3(str_param: str = "default", int_param: int = 0) -> None:
        return None

    @decorator(["/handler4/int/{int_param:int}", "/handler4/str/{str_param:str}"], name="handler4")  # type: ignore[call-arg]
    def handler4(int_param: int = 1, str_param: str = "str") -> None:
        return None

    router = Router("router-path/", route_handlers=[handler, handler_nameless, handler_no_params, handler3, handler4])
    router_with_param = Router("router-with-param/{router_param:str}", route_handlers=[handler2])
    app = Litestar(route_handlers=[router, router_with_param])

    reversed_url_path = app.route_reverse("handler-name", param="param-value")
    assert reversed_url_path == "/router-path/path-one/param-value"

    reversed_url_path = app.route_reverse(handler_nameless)
    assert reversed_url_path == "/router-path/path-nameless"

    reversed_url_path = app.route_reverse("handler-no-params")
    assert reversed_url_path == "/router-path/path-two"

    reversed_url_path = app.route_reverse(
        "multiple-params-handler-name", router_param="router", str_param="abc", int_param=123
    )
    assert reversed_url_path == "/router-with-param/router/multiple/abc/params/123"

    reversed_url_path = app.route_reverse("handler4", int_param=100)
    assert reversed_url_path == "/router-path/handler4/int/100"

    reversed_url_path = app.route_reverse("handler4", str_param="string")
    assert reversed_url_path == "/router-path/handler4/str/string"

    with pytest.raises(NoRouteMatchFoundException):
        reversed_url_path = app.route_reverse("nonexistent-handler")


@pytest.mark.parametrize(
    "complex_path_param",
    [("time", time(hour=14), "14:00"), ("float", float(1 / 3), "0.33")],
)
def test_route_reverse_validation_complex_params(complex_path_param) -> None:  # type: ignore[no-untyped-def]
    param_type, param_value, param_manual_str = complex_path_param

    @get(f"/abc/{{param:{param_type}}}", name="handler")
    def handler() -> None:
        pass

    app = Litestar(route_handlers=[handler])

    # test that complex types of path params accept either itself
    # or string but nothing else
    with pytest.raises(NoRouteMatchFoundException):
        app.route_reverse("handler", param=123)

    reversed_url_path = app.route_reverse("handler", param=param_manual_str)
    assert reversed_url_path == f"/abc/{param_manual_str}"

    reversed_url_path = app.route_reverse("handler", param=param_value)
    assert reversed_url_path == f"/abc/{param_value}"


def test_route_reverse_validation() -> None:
    @get("/abc/{param:int}", name="handler-name")
    def handler_one() -> None:
        pass

    @get("/def/{param:str}", name="another-handler-name")
    def handler_two() -> None:
        pass

    app = Litestar(route_handlers=[handler_one, handler_two])

    with pytest.raises(NoRouteMatchFoundException):
        app.route_reverse("handler-name")

    with pytest.raises(NoRouteMatchFoundException):
        app.route_reverse("handler-name", param="str")

    with pytest.raises(NoRouteMatchFoundException):
        app.route_reverse("another-handler-name", param=1)


def test_route_reverse_allow_string_params() -> None:
    @get(
        "/strings-everywhere/{datetime_param:datetime}/{date_param:date}/"
        "{time_param:time}/{timedelta_param:timedelta}/"
        "{float_param:float}/{uuid_param:uuid}/{path_param:path}",
        name="strings-everywhere-handler",
    )
    def strings_everywhere_handler(
        datetime_param: datetime,
        date_param: date,
        time_param: time,
        timedelta_param: timedelta,
        float_param: float,
        uuid_param: UUID,
        path_param: Path,
    ) -> None:
        return None

    app = Litestar(route_handlers=[strings_everywhere_handler])

    reversed_url_path = app.route_reverse(
        "strings-everywhere-handler",
        datetime_param="0001-01-01T01:01:01.000001Z",  # datetime(1, 1, 1, 1, 1, 1, 1, tzinfo=UTC)
        date_param="0001-01-01",  # date(1,1,1)
        time_param="01:01:01.000001Z",  # time(1, 1, 1, 1, tzinfo=UTC)
        timedelta_param="P8DT3661.001001S",  # timedelta(1, 1, 1, 1, 1, 1, 1)
        float_param="0.1",
        uuid_param="00000000-0000-0000-0000-000000000000",  # UUID(int=0)
        path_param="/home/user",  # Path("/home/user/"),
    )

    assert (
        reversed_url_path == "/strings-everywhere/0001-01-01T01:01:01.000001Z/"
        "0001-01-01/01:01:01.000001Z/P8DT3661.001001S/0.1/"
        "00000000-0000-0000-0000-000000000000/home/user"
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.utils import warn_deprecation

__all__ = (
    "ModelT",
    "SQLAlchemyAsyncRepository",
    "SQLAlchemySyncRepository",
    "wrap_sqlalchemy_exception",
)


def __getattr__(attr_name: str) -> object:
    if attr_name in __all__:
        if attr_name in ("SQLAlchemyAsyncRepository", "SQLAlchemySyncRepository", "ModelT"):
            module = "litestar.plugins.sqlalchemy.repository"
            from advanced_alchemy.extensions.litestar import (
                repository,  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
            )

            value = globals()[attr_name] = getattr(repository, attr_name)
        elif attr_name == "wrap_sqlalchemy_exception":
            module = "litestar.plugins.sqlalchemy.exceptions"
            from advanced_alchemy.extensions.litestar import (
                exceptions,  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
            )

            value = globals()[attr_name] = getattr(exceptions, attr_name)

        else:  # pragma: no cover
            raise RuntimeError(f"Unhandled module attribute: {attr_name!r}")

        warn_deprecation(
            deprecated_name=f"litestar.contrib.sqlalchemy.{attr_name}",
            version="2.12",
            kind="import",
            removal_in="3.0",
            info=f"importing {attr_name} from 'litestar.contrib.sqlalchemy' is deprecated, please "
            f"import it from '{module}' instead",
        )

        return value

    raise AttributeError(f"module {__name__!r} has no attribute {attr_name!r}")  # pragma: no cover


if TYPE_CHECKING:
    from advanced_alchemy.exceptions import (  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
        wrap_sqlalchemy_exception,
    )
    from advanced_alchemy.repository import (  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
        ModelT,
        SQLAlchemyAsyncRepository,
        SQLAlchemySyncRepository,
    )



from typing import Any, Dict

from litestar import Litestar, get
from litestar.datastructures import ImmutableState


@get("/", sync_to_thread=False)
def handler(state: ImmutableState) -> Dict[str, Any]:
    setattr(state, "count", 1)  # raises AttributeError
    return state.dict()


app = Litestar(route_handlers=[handler])



from litestar.status_codes import HTTP_201_CREATED
from litestar.testing.client import TestClient


def test_create_user(user_data: dict) -> None:
    from docs.examples.data_transfer_objects.overriding_implicit_return_dto import app

    with TestClient(app=app) as client:
        response = client.post("/", json=user_data)

    assert response.status_code == HTTP_201_CREATED
    assert response.content == b"Mr Sunglass"



import anyio

from litestar import Litestar
from litestar.middleware import AbstractMiddleware, DefineMiddleware
from litestar.types import ASGIApp, Receive, Scope, Scopes, Send


class TimeoutMiddleware(AbstractMiddleware):
    def __init__(
        self,
        app: ASGIApp,
        timeout: float,
        exclude: str | list[str] | None = None,
        exclude_opt_key: str | None = None,
        scopes: Scopes | None = None,
    ):
        self.timeout = timeout
        super().__init__(app=app, exclude=exclude, exclude_opt_key=exclude_opt_key, scopes=scopes)

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        with anyio.move_on_after(self.timeout):
            await self.app(scope, receive, send)


app = Litestar(
    middleware=[
        DefineMiddleware(
            TimeoutMiddleware,
            timeout=5,
        )
    ]
)



from typing import Dict

from litestar import Litestar, Request, get
from litestar.logging.config import StructLoggingConfig
from litestar.middleware.logging import LoggingMiddlewareConfig


@get("/")
async def handler(request: Request) -> Dict[str, str]:
    request.logger.info("Logging in the handler")
    return {"hello": "world"}


logging_middleware_config = LoggingMiddlewareConfig()

app = Litestar(
    route_handlers=[handler],
    logging_config=StructLoggingConfig(log_exceptions="always"),
    middleware=[logging_middleware_config.middleware],
)


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app)



import secrets
from os import environ
from typing import Any, Dict, Optional
from uuid import UUID

from pydantic import BaseModel, EmailStr

from litestar import Litestar, Request, Response, get, post
from litestar.connection import ASGIConnection
from litestar.openapi.config import OpenAPIConfig
from litestar.security.jwt import JWTCookieAuth, Token


# Let's assume we have a User model that is a pydantic model.
# This though is not required - we need some sort of user class -
# but it can be any arbitrary value, e.g. an SQLAlchemy model, a representation of a MongoDB  etc.
class User(BaseModel):
    id: UUID
    name: str
    email: EmailStr


MOCK_DB: Dict[str, User] = {}


# JWTCookieAuth requires a retrieve handler callable that receives the JWT token model and the ASGI connection
# and returns the 'User' instance correlating to it.
#
# Notes:
# - 'User' can be any arbitrary value you decide upon.
# - The callable can be either sync or async - both will work.
async def retrieve_user_handler(token: "Token", connection: "ASGIConnection[Any, Any, Any, Any]") -> Optional[User]:
    # logic here to retrieve the user instance
    return MOCK_DB.get(token.sub)


jwt_cookie_auth = JWTCookieAuth[User](
    retrieve_user_handler=retrieve_user_handler,
    token_secret=environ.get("JWT_SECRET", secrets.token_hex()),
    # we are specifying which endpoints should be excluded from authentication. In this case the login endpoint
    # and our openAPI docs.
    exclude=["/login", "/schema"],
    # Tip: We can optionally supply cookie options to the configuration.  Here is an example of enabling the secure cookie option
    # secure=True,
)


# Given an instance of 'JWTCookieAuth' we can create a login handler function:
@post("/login")
async def login_handler(data: "User") -> "Response[User]":
    MOCK_DB[str(data.id)] = data
    return jwt_cookie_auth.login(identifier=str(data.id), response_body=data)


# We also have some other routes, for example:
@get("/some-path", sync_to_thread=False)
def some_route_handler(request: "Request[User, Token, Any]") -> Any:
    # request.user is set to the instance of user returned by the middleware
    assert isinstance(request.user, User)
    # request.auth is the instance of 'litestar_jwt.Token' created from the data encoded in the auth header
    assert isinstance(request.auth, Token)
    # do stuff ...


# We create our OpenAPIConfig as usual - the JWT security scheme will be injected into it.
openapi_config = OpenAPIConfig(
    title="My API",
    version="1.0.0",
)

# We initialize the app instance and pass the jwt_cookie_auth 'on_app_init' handler to the constructor.
# The hook handler will inject the JWT middleware and openapi configuration into the app.
app = Litestar(
    route_handlers=[login_handler, some_route_handler],
    on_app_init=[jwt_cookie_auth.on_app_init],
    openapi_config=openapi_config,
)



# ruff: noqa: F401
# pyright: reportUnusedImport=false
from __future__ import annotations

from typing import TYPE_CHECKING

from litestar.utils import warn_deprecation

__all__ = (
    "ModelT",
    "RowT",
    "SQLAlchemyAsyncRepositoryT",
    "SQLAlchemySyncRepositoryT",
    "SelectT",
)


def __getattr__(attr_name: str) -> object:
    if attr_name in __all__:
        from advanced_alchemy.repository.typing import (  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
            ModelT,
            RowT,
            SelectT,
            SQLAlchemyAsyncRepositoryT,
            SQLAlchemySyncRepositoryT,
        )

        warn_deprecation(
            deprecated_name=f"litestar.contrib.sqlalchemy.repository.types.{attr_name}",
            version="2.12",
            kind="import",
            removal_in="3.0",
            info=f"importing {attr_name} from 'litestar.contrib.sqlalchemy.repository.types' is deprecated, please "
            f"import it from 'litestar.plugins.sqlalchemy.repository.typing' instead",
        )
        value = globals()[attr_name] = locals()[attr_name]
        return value

    raise AttributeError(f"module {__name__!r} has no attribute {attr_name!r}")  # pragma: no cover


if TYPE_CHECKING:
    from advanced_alchemy.repository.typing import (  # type: ignore[import-not-found] # pyright: ignore[reportMissingImports]
        ModelT,
        RowT,
        SelectT,
        SQLAlchemyAsyncRepositoryT,
        SQLAlchemySyncRepositoryT,
    )



from typing import TYPE_CHECKING, List, Tuple, Union
from unittest.mock import MagicMock
from warnings import catch_warnings

import pytest

from litestar import MediaType, asgi, get
from litestar.datastructures.headers import MutableScopeHeaders
from litestar.exceptions import LitestarWarning, ValidationException
from litestar.middleware import AbstractMiddleware, ASGIMiddleware, DefineMiddleware
from litestar.response.base import ASGIResponse
from litestar.status_codes import HTTP_400_BAD_REQUEST
from litestar.testing import create_test_client

if TYPE_CHECKING:
    from litestar.types import ASGIApp, Message, Receive, Scope, Send


def test_custom_middleware() -> None:
    class SubclassMiddleware(AbstractMiddleware):
        async def __call__(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
            async def _send(message: "Message") -> None:
                if message["type"] == "http.response.start":
                    headers = MutableScopeHeaders(message)
                    headers.add("test", str(123))
                await send(message)

            await self.app(scope, receive, _send)

    @get("/")
    def handler() -> dict:
        return {"hello": "world"}

    with create_test_client(handler, middleware=[DefineMiddleware(SubclassMiddleware)]) as client:
        response = client.get("/")
        assert response.headers["test"] == "123"


def test_raises_exception() -> None:
    class SubclassMiddleware(AbstractMiddleware):
        async def __call__(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
            raise ValidationException(detail="nope")

    @get("/")
    def handler() -> dict:
        return {"hello": "world"}

    with create_test_client(handler, middleware=[DefineMiddleware(SubclassMiddleware)]) as client:
        response = client.get("/")
        assert response.status_code == HTTP_400_BAD_REQUEST


def test_exclude_by_pattern() -> None:
    class SubclassMiddleware(AbstractMiddleware):
        exclude = r"^/123"

        async def __call__(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
            async def _send(message: "Message") -> None:
                if message["type"] == "http.response.start":
                    headers = MutableScopeHeaders(message)
                    headers.add("test", str(123))
                await send(message)

            await self.app(scope, receive, _send)

    @get("/123")
    def first_handler() -> dict:
        return {"hello": "world"}

    @get("/456")
    def second_handler() -> dict:
        return {"hello": "world"}

    @asgi("/mount", is_mount=True)
    async def handler(scope: "Scope", receive: "Receive", send: "Send") -> None:
        response = ASGIResponse(body=b"ok", media_type=MediaType.TEXT)
        await response(scope, receive, send)

    with create_test_client(
        [first_handler, second_handler, handler], middleware=[DefineMiddleware(SubclassMiddleware)]
    ) as client:
        response = client.get("/123")
        assert "test" not in response.headers

        response = client.get("/456")
        assert "test" in response.headers

        response = client.get("/mount/123")
        assert "test" in response.headers


def test_exclude_by_pattern_list() -> None:
    class SubclassMiddleware(AbstractMiddleware):
        exclude = ["123", "456"]

        async def __call__(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
            async def _send(message: "Message") -> None:
                if message["type"] == "http.response.start":
                    headers = MutableScopeHeaders(message)
                    headers.add("test", str(123))
                await send(message)

            await self.app(scope, receive, _send)

    @get("/123")
    def first_handler() -> dict:
        return {"hello": "world"}

    @get("/456")
    def second_handler() -> dict:
        return {"hello": "world"}

    @get("/789")
    def third_handler() -> dict:
        return {"hello": "world"}

    with create_test_client(
        [first_handler, second_handler, third_handler], middleware=[DefineMiddleware(SubclassMiddleware)]
    ) as client:
        response = client.get("/123")
        assert "test" not in response.headers
        response = client.get("/456")
        assert "test" not in response.headers
        response = client.get("/789")
        assert "test" in response.headers


@pytest.mark.parametrize("excludes", ["/", ["/", "/foo"], "/*", "/.*"])
def test_exclude_by_pattern_warns_if_exclude_all(excludes: Union[str, List[str]]) -> None:
    class SubclassMiddleware(AbstractMiddleware):
        exclude = excludes

        async def __call__(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
            await self.app(scope, receive, send)

    with pytest.warns(LitestarWarning, match="Middleware 'SubclassMiddleware' exclude pattern"):
        create_test_client(middleware=[SubclassMiddleware])


def test_exclude_doesnt_warn_on_non_greedy_pattern() -> None:
    class SubclassMiddleware(AbstractMiddleware):
        exclude = "^/$"

        async def __call__(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
            await self.app(scope, receive, send)

    with catch_warnings(record=True) as warnings:
        create_test_client(middleware=[SubclassMiddleware])
        assert len(warnings) == 0


def test_exclude_by_opt_key() -> None:
    class SubclassMiddleware(AbstractMiddleware):
        exclude_opt_key = "exclude_route"

        async def __call__(self, scope: "Scope", receive: "Receive", send: "Send") -> None:
            async def _send(message: "Message") -> None:
                if message["type"] == "http.response.start":
                    headers = MutableScopeHeaders(message)
                    headers.add("test", str(123))
                await send(message)

                await self.app(scope, receive, _send)

    @get("/", exclude_route=True)
    def handler() -> dict:
        return {"hello": "world"}

    with create_test_client(handler, middleware=[DefineMiddleware(SubclassMiddleware)]) as client:
        response = client.get("/")
        assert "test" not in response.headers


def test_abstract_middleware_deprecation_warning() -> None:
    with pytest.warns(DeprecationWarning, match="AbstractMiddleware"):

        class MyMiddleware(AbstractMiddleware):
            pass


def test_asgi_middleware() -> None:
    class SubclassMiddleware(ASGIMiddleware):
        async def handle(self, scope: "Scope", receive: "Receive", send: "Send", next_app: "ASGIApp") -> None:
            async def _send(message: "Message") -> None:
                if message["type"] == "http.response.start":
                    headers = MutableScopeHeaders(message)
                    headers.add("test", str(123))
                await send(message)

            await next_app(scope, receive, _send)

    @get("/")
    def handler() -> dict:
        return {"hello": "world"}

    with create_test_client(handler, middleware=[SubclassMiddleware()]) as client:
        response = client.get("/")
        assert response.headers["test"] == "123"


def test_asgi_middleware_raises_exception() -> None:
    class SubclassMiddleware(ASGIMiddleware):
        async def handle(self, scope: "Scope", receive: "Receive", send: "Send", next_app: "ASGIApp") -> None:
            raise ValidationException(detail="nope")

    @get("/")
    def handler() -> dict:
        return {"hello": "world"}

    with create_test_client(handler, middleware=[SubclassMiddleware()]) as client:
        response = client.get("/")
        assert response.status_code == HTTP_400_BAD_REQUEST


def test_asgi_middleware_exclude_by_pattern() -> None:
    class SubclassMiddleware(ASGIMiddleware):
        def __init__(self) -> None:
            self.exclude_path_pattern = r"^/123"

        async def handle(self, scope: "Scope", receive: "Receive", send: "Send", next_app: "ASGIApp") -> None:
            async def _send(message: "Message") -> None:
                if message["type"] == "http.response.start":
                    headers = MutableScopeHeaders(message)
                    headers.add("test", str(123))
                await send(message)

            await next_app(scope, receive, _send)

    @get("/123")
    def first_handler() -> dict:
        return {"hello": "world"}

    @get("/456")
    def second_handler() -> dict:
        return {"hello": "world"}

    @asgi("/mount", is_mount=True)
    async def handler(scope: "Scope", receive: "Receive", send: "Send") -> None:
        response = ASGIResponse(body=b"ok", media_type=MediaType.TEXT)
        await response(scope, receive, send)

    with create_test_client([first_handler, second_handler, handler], middleware=[SubclassMiddleware()]) as client:
        response = client.get("/123")
        assert "test" not in response.headers

        response = client.get("/456")
        assert "test" in response.headers

        response = client.get("/mount/123")
        assert "test" in response.headers


def test_asgi_middleware_exclude_by_pattern_tuple() -> None:
    class SubclassMiddleware(ASGIMiddleware):
        exclude_path_pattern = ("123", "456")

        async def handle(self, scope: "Scope", receive: "Receive", send: "Send", next_app: "ASGIApp") -> None:
            async def _send(message: "Message") -> None:
                if message["type"] == "http.response.start":
                    headers = MutableScopeHeaders(message)
                    headers.add("test", str(123))
                await send(message)

            await next_app(scope, receive, _send)

    @get("/123")
    def first_handler() -> dict:
        return {"hello": "world"}

    @get("/456")
    def second_handler() -> dict:
        return {"hello": "world"}

    @get("/789")
    def third_handler() -> dict:
        return {"hello": "world"}

    with create_test_client(
        [first_handler, second_handler, third_handler], middleware=[SubclassMiddleware()]
    ) as client:
        response = client.get("/123")
        assert "test" not in response.headers
        response = client.get("/456")
        assert "test" not in response.headers
        response = client.get("/789")
        assert "test" in response.headers


def test_asgi_middleware_should_exclude_scope() -> None:
    mock = MagicMock()

    class SubclassMiddleware(ASGIMiddleware):
        @staticmethod
        def should_bypass_for_scope(scope: "Scope") -> bool:
            return scope["path"].endswith(".jpg")

        async def handle(self, scope: "Scope", receive: "Receive", send: "Send", next_app: "ASGIApp") -> None:
            mock(scope["path"])
            await next_app(scope, receive, send)

    @get("/{file_name:str}")
    def handler(file_name: str) -> str:
        return file_name

    with create_test_client([handler], middleware=[SubclassMiddleware()]) as client:
        assert client.get("/test.txt").status_code == 200
        assert client.get("/test.jpg").status_code == 200

        mock.assert_called_once_with("/test.txt")


def test_asgi_middleware_path_exclude_warns_future_use() -> None:
    mock = MagicMock()

    class SubclassMiddleware(ASGIMiddleware):
        def __init__(self, pattern: str) -> None:
            self.exclude_path_pattern = pattern

        async def handle(self, scope: "Scope", receive: "Receive", send: "Send", next_app: "ASGIApp") -> None:
            mock(scope["path"])
            await next_app(scope, receive, send)

    @get("/{file_name:str}")
    def handler(file_name: str) -> str:
        return file_name

    # this configuration would NOT be excluded in the future
    with create_test_client([handler], middleware=[SubclassMiddleware(".jpg")]) as client:
        with pytest.warns(DeprecationWarning, match=".*exclude_path_pattern.* did match the request path"):
            assert client.get("/test.jpg").status_code == 200

        assert client.get("/test.txt").status_code == 200
        mock.assert_called_once_with("/test.txt")

    mock.reset_mock()

    # this configuration WOULD be excluded in the future
    with create_test_client([handler], middleware=[SubclassMiddleware("str")]) as client:
        with pytest.warns(DeprecationWarning, match=".*exclude_path_pattern.* did not match the request path"):
            assert client.get("/test.jpg").status_code == 200
            mock.assert_called_once_with("/test.jpg")


@pytest.mark.parametrize("excludes", ["/", ("/", "/foo"), "/*", "/.*"])
def test_asgi_middleware_exclude_by_pattern_warns_if_exclude_all(excludes: Union[str, Tuple[str, ...]]) -> None:
    class SubclassMiddleware(ASGIMiddleware):
        exclude_path_pattern = excludes

        async def handle(self, scope: "Scope", receive: "Receive", send: "Send", next_app: "ASGIApp") -> None:
            await next_app(scope, receive, send)

    with pytest.warns(LitestarWarning, match="Middleware 'SubclassMiddleware' exclude pattern"):
        create_test_client(middleware=[SubclassMiddleware()])


def test_asgi_middleware_exclude_doesnt_warn_on_non_greedy_pattern() -> None:
    class SubclassMiddleware(ASGIMiddleware):
        exclude_path_pattern = "^/$"

        async def handle(self, scope: "Scope", receive: "Receive", send: "Send", next_app: "ASGIApp") -> None:
            await next_app(scope, receive, send)

    with catch_warnings(record=True) as warnings:
        create_test_client(middleware=[SubclassMiddleware()])
        assert len(warnings) == 0


def test_asgi_middleware_exclude_by_opt_key() -> None:
    class SubclassMiddleware(ASGIMiddleware):
        exclude_opt_key = "exclude_route"

        async def handle(self, scope: "Scope", receive: "Receive", send: "Send", next_app: "ASGIApp") -> None:
            async def _send(message: "Message") -> None:
                if message["type"] == "http.response.start":
                    headers = MutableScopeHeaders(message)
                    headers.add("test", str(123))
                await send(message)

                await next_app(scope, receive, send)

    @get("/", exclude_route=True)
    def handler() -> dict:
        return {"hello": "world"}

    with create_test_client(handler, middleware=[SubclassMiddleware()]) as client:
        response = client.get("/")
        assert "test" not in response.headers



# mypy: strict-equality=False
# pyright: reportGeneralTypeIssues=false
from __future__ import annotations

import datetime
import re
from dataclasses import dataclass
from inspect import isclass
from typing import TYPE_CHECKING, Any, Callable, Literal, Optional

from typing_extensions import Annotated, NotRequired, get_type_hints

from litestar.openapi.spec import Example
from litestar.params import KwargDefinition, ParameterKwarg
from litestar.types import Empty
from litestar.typing import FieldDefinition
from litestar.utils import deprecated, is_class_and_subclass, is_generic, is_undefined_sentinel
from litestar.utils.typing import (
    _substitute_typevars,
    get_origin_or_inner_type,
    get_safe_generic_origin,
    get_type_hints_with_generics_resolved,
    normalize_type_annotation,
)

# isort: off
try:
    from pydantic import v1 as pydantic_v1
    import pydantic as pydantic_v2
    from pydantic.fields import PydanticUndefined as Pydantic2Undefined  # type: ignore[attr-defined]
    from pydantic.v1.fields import Undefined as Pydantic1Undefined

    PYDANTIC_UNDEFINED_SENTINELS = {Pydantic1Undefined, Pydantic2Undefined}
except ImportError:
    try:
        import pydantic as pydantic_v1  # type: ignore[no-redef]
        from pydantic.fields import Undefined as Pydantic1Undefined  # type: ignore[attr-defined, no-redef]

        pydantic_v2 = Empty  # type: ignore[assignment]
        PYDANTIC_UNDEFINED_SENTINELS = {Pydantic1Undefined}

    except ImportError:  # pyright: ignore
        pydantic_v1 = Empty  # type: ignore[assignment]
        pydantic_v2 = Empty  # type: ignore[assignment]
        PYDANTIC_UNDEFINED_SENTINELS = set()
# isort: on


if TYPE_CHECKING:
    from types import ModuleType

    from typing_extensions import TypeGuard


def is_pydantic_model_class(
    annotation: Any,
) -> TypeGuard[type[pydantic_v1.BaseModel | pydantic_v2.BaseModel]]:  # pyright: ignore
    """Given a type annotation determine if the annotation is a subclass of pydantic's BaseModel.

    Args:
        annotation: A type.

    Returns:
        A typeguard determining whether the type is :data:`BaseModel pydantic.BaseModel>`.
    """
    tests: list[bool] = []

    if pydantic_v1 is not Empty:  # pragma: no cover
        tests.append(is_class_and_subclass(annotation, pydantic_v1.BaseModel))

    if pydantic_v2 is not Empty:  # pragma: no cover
        tests.append(is_class_and_subclass(annotation, pydantic_v2.BaseModel))

    return any(tests)


def is_pydantic_model_instance(
    annotation: Any,
) -> TypeGuard[pydantic_v1.BaseModel | pydantic_v2.BaseModel]:  # pyright: ignore
    """Given a type annotation determine if the annotation is an instance of pydantic's BaseModel.

    Args:
        annotation: A type.

    Returns:
        A typeguard determining whether the type is :data:`BaseModel pydantic.BaseModel>`.
    """
    tests: list[bool] = []

    if pydantic_v1 is not Empty:  # pragma: no cover
        tests.append(isinstance(annotation, pydantic_v1.BaseModel))

    if pydantic_v2 is not Empty:  # pragma: no cover
        tests.append(isinstance(annotation, pydantic_v2.BaseModel))

    return any(tests)


def is_pydantic_constrained_field(annotation: Any) -> bool:
    """Check if the given annotation is a constrained pydantic type.

    Args:
        annotation: A type annotation

    Returns:
        True if pydantic is installed and the type is a constrained type, otherwise False.
    """
    if pydantic_v1 is Empty:  # pragma: no cover
        return False  # type: ignore[unreachable]

    return any(
        is_class_and_subclass(annotation, constrained_type)  # pyright: ignore
        for constrained_type in (
            pydantic_v1.ConstrainedBytes,
            pydantic_v1.ConstrainedDate,
            pydantic_v1.ConstrainedDecimal,
            pydantic_v1.ConstrainedFloat,
            pydantic_v1.ConstrainedFrozenSet,
            pydantic_v1.ConstrainedInt,
            pydantic_v1.ConstrainedList,
            pydantic_v1.ConstrainedSet,
            pydantic_v1.ConstrainedStr,
        )
    )


def pydantic_unwrap_and_get_origin(annotation: Any) -> Any | None:
    if pydantic_v2 is Empty or (pydantic_v1 is not Empty and is_class_and_subclass(annotation, pydantic_v1.BaseModel)):
        return get_origin_or_inner_type(annotation)

    origin = annotation.__pydantic_generic_metadata__["origin"]
    return normalize_type_annotation(origin)


def pydantic_get_type_hints_with_generics_resolved(
    annotation: Any,
    globalns: dict[str, Any] | None = None,
    localns: dict[str, Any] | None = None,
    include_extras: bool = False,
    model_annotations: dict[str, Any] | None = None,
) -> dict[str, Any]:
    if pydantic_v2 is Empty or (pydantic_v1 is not Empty and is_class_and_subclass(annotation, pydantic_v1.BaseModel)):
        return get_type_hints_with_generics_resolved(annotation, type_hints=model_annotations)

    origin = pydantic_unwrap_and_get_origin(annotation)
    if origin is None:
        if model_annotations is None:  # pragma: no cover
            model_annotations = get_type_hints(
                annotation, globalns=globalns, localns=localns, include_extras=include_extras
            )
        typevar_map = {p: p for p in annotation.__pydantic_generic_metadata__["parameters"]}
    else:
        if model_annotations is None:
            model_annotations = get_type_hints(
                origin, globalns=globalns, localns=localns, include_extras=include_extras
            )
        args = annotation.__pydantic_generic_metadata__["args"]
        parameters = origin.__pydantic_generic_metadata__["parameters"]
        typevar_map = dict(zip(parameters, args))

    return {n: _substitute_typevars(type_, typevar_map) for n, type_ in model_annotations.items()}


@deprecated(version="2.6.2")
def pydantic_get_unwrapped_annotation_and_type_hints(annotation: Any) -> tuple[Any, dict[str, Any]]:  # pragma: no cover
    """Get the unwrapped annotation and the type hints after resolving generics.

    Args:
        annotation: A type annotation.

    Returns:
        A tuple containing the unwrapped annotation and the type hints.
    """

    if is_generic(annotation):
        origin = pydantic_unwrap_and_get_origin(annotation)
        return origin or annotation, pydantic_get_type_hints_with_generics_resolved(annotation, include_extras=True)
    return annotation, get_type_hints(annotation, include_extras=True)


def is_pydantic_2_model(
    obj: type[pydantic_v1.BaseModel | pydantic_v2.BaseModel],  # pyright: ignore
) -> TypeGuard[pydantic_v2.BaseModel]:  # pyright: ignore
    return pydantic_v2 is not Empty and issubclass(obj, pydantic_v2.BaseModel)


def is_pydantic_undefined(value: Any) -> bool:
    return any(v is value for v in PYDANTIC_UNDEFINED_SENTINELS)


def create_field_definitions_for_computed_fields(
    model: type[pydantic_v1.BaseModel | pydantic_v2.BaseModel],  # pyright: ignore
    prefer_alias: bool,
) -> dict[str, FieldDefinition]:
    """Create field definitions for computed fields.

    Args:
        model: A pydantic model.
        prefer_alias: Whether to prefer the alias or the name of the field.

    Returns:
        A dictionary containing the field definitions for the computed fields.
    """
    pydantic_decorators = getattr(model, "__pydantic_decorators__", None)
    if pydantic_decorators is None:
        return {}

    def get_name(k: str, dec: Any) -> str:
        if not dec.info.alias:
            return k
        return dec.info.alias if prefer_alias else k  # type: ignore[no-any-return]

    return {
        (name := get_name(k, dec)): FieldDefinition.from_annotation(
            Annotated[
                dec.info.return_type,
                KwargDefinition(
                    title=dec.info.title,
                    description=dec.info.description,
                    read_only=True,
                    examples=[Example(value=v) for v in examples] if (examples := dec.info.examples) else None,
                    schema_extra=dec.info.json_schema_extra,
                ),
            ],
            name=name,
        )
        for k, dec in pydantic_decorators.computed_fields.items()
    }


def is_pydantic_v2(module: ModuleType) -> bool:
    """Determine if the given module is pydantic v2.

    Given a module we expect to be a pydantic version, determine if it is pydantic v2.

    Args:
        module: A module.

    Returns:
        True if the module is pydantic v2, otherwise False.
    """
    return bool(module.__version__.startswith("2."))


@dataclass(frozen=True)
class PydanticModelInfo:
    pydantic_version: Literal["1", "2"]
    field_definitions: dict[str, FieldDefinition]
    model_fields: dict[str, pydantic_v1.fields.FieldInfo | pydantic_v2.fields.FieldInfo]  # pyright: ignore[reportInvalidTypeForm,reportGeneralTypeIssues]
    title: str | None = None
    example: Any | None = None
    is_generic: bool = False


_CreateFieldDefinition = Callable[..., FieldDefinition]


def _create_field_definition_v1(  # noqa: C901
    field_annotation: Any,
    *,
    field_info: pydantic_v1.fields.FieldInfo,  # pyright: ignore[reportInvalidTypeForm,reportGeneralTypeIssues]
    **field_definition_kwargs: Any,
) -> FieldDefinition:
    kwargs: dict[str, Any] = {}
    examples: list[Any] = []
    if example := field_info.extra.get("example"):
        examples.append(example)
    if extra_examples := field_info.extra.get("examples"):
        examples.extend(extra_examples)
    if examples:
        kwargs["examples"] = [Example(value=e) for e in examples]
    if title := field_info.title:
        kwargs["title"] = title
    if description := field_info.description:
        kwargs["description"] = description

    kwarg_definition: KwargDefinition | None = None

    if isclass(field_annotation):
        if issubclass(field_annotation, pydantic_v1.ConstrainedBytes):  # pyright: ignore[reportArgumentType,reportAttributeAccessIssue]
            kwarg_definition = ParameterKwarg(
                min_length=field_annotation.min_length,
                max_length=field_annotation.max_length,
                lower_case=field_annotation.to_lower,
                upper_case=field_annotation.to_upper,
                **kwargs,
            )
            field_definition_kwargs["raw"] = field_annotation
            field_annotation = bytes
        elif issubclass(field_annotation, pydantic_v1.ConstrainedStr):  # pyright: ignore[reportArgumentType,reportAttributeAccessIssue]
            kwarg_definition = ParameterKwarg(
                min_length=field_annotation.min_length,
                max_length=field_annotation.max_length,
                lower_case=field_annotation.to_lower,
                upper_case=field_annotation.to_upper,
                pattern=field_annotation.regex.pattern
                if isinstance(field_annotation.regex, re.Pattern)
                else field_annotation.regex,
                **kwargs,
            )
            field_definition_kwargs["raw"] = field_annotation
            field_annotation = str
        elif issubclass(field_annotation, pydantic_v1.ConstrainedDate):  # pyright: ignore[reportArgumentType,reportAttributeAccessIssue]
            # TODO: The typings of ParameterKwarg need fixing. Specifically, the
            # gt/ge/lt/le fields need to be typed with protocols, such that they may
            # accept any type that implements the respective comparisons

            kwarg_definition = ParameterKwarg(
                gt=field_annotation.gt,  # type: ignore[arg-type]
                ge=field_annotation.ge,  # type: ignore[arg-type]
                lt=field_annotation.lt,  # type: ignore[arg-type]
                le=field_annotation.le,  # type: ignore[arg-type]
                **kwargs,
            )
            field_definition_kwargs["raw"] = field_annotation
            field_annotation = datetime.date
        elif issubclass(
            field_annotation,
            (pydantic_v1.ConstrainedInt, pydantic_v1.ConstrainedFloat, pydantic_v1.ConstrainedDecimal),  # pyright: ignore[reportArgumentType,reportAttributeAccessIssue]
        ):
            kwarg_definition = ParameterKwarg(
                gt=field_annotation.gt,  # type: ignore[arg-type]
                ge=field_annotation.ge,  # type: ignore[arg-type]
                lt=field_annotation.lt,  # type: ignore[arg-type]
                le=field_annotation.le,  # type: ignore[arg-type]
                multiple_of=field_annotation.multiple_of,  # type: ignore[arg-type]
                **kwargs,
            )
            field_definition_kwargs["raw"] = field_annotation
            field_annotation = field_annotation.mro()[2]
        elif issubclass(
            field_annotation,
            (pydantic_v1.ConstrainedList, pydantic_v1.ConstrainedSet, pydantic_v1.ConstrainedFrozenSet),  # pyright: ignore[reportArgumentType,reportAttributeAccessIssue]
        ):
            kwarg_definition = ParameterKwarg(
                max_items=field_annotation.max_items, min_items=field_annotation.min_items, **kwargs
            )
            field_definition_kwargs["raw"] = field_annotation
            # on < 3.9, these builtins are not generic
            origin = get_safe_generic_origin(None, field_annotation.__origin__)
            field_annotation = origin[field_annotation.item_type]

    if kwarg_definition is None and kwargs:
        kwarg_definition = ParameterKwarg(**kwargs)

    if kwarg_definition:
        field_definition_kwargs["raw"] = field_annotation
        field_annotation = Annotated[field_annotation, kwarg_definition]

    return FieldDefinition.from_annotation(
        annotation=field_annotation,
        **field_definition_kwargs,
    )


def _create_field_definition_v2(  # noqa: C901
    field_annotation: Any,
    *,
    field_info: pydantic_v2.fields.FieldInfo,  # pyright: ignore[reportInvalidTypeForm,reportGeneralTypeIssues]
    **field_definition_kwargs: Any,
) -> FieldDefinition:
    kwargs: dict[str, Any] = {}
    examples: list[Any] = []
    field_meta: list[Any] = []

    if json_schema_extra := field_info.json_schema_extra:
        if callable(json_schema_extra):
            raise ValueError("Callables not supported for json_schema_extra")
        if json_schema_example := json_schema_extra.get("example"):
            del json_schema_extra["example"]
            examples.append(json_schema_example)
        if json_schema_examples := json_schema_extra.get("examples"):
            del json_schema_extra["examples"]
            examples.extend(json_schema_examples)  # type: ignore[arg-type]
    if field_examples := field_info.examples:
        examples.extend(field_examples)

    if examples:
        if not json_schema_extra:
            json_schema_extra = {}
        json_schema_extra["examples"] = examples

    if description := field_info.description:
        kwargs["description"] = description

    if title := field_info.title:
        kwargs["title"] = title

    for meta in field_info.metadata:
        if isinstance(meta, pydantic_v2.types.StringConstraints):  # pyright: ignore[reportAttributeAccessIssue]
            kwargs["min_length"] = meta.min_length
            kwargs["max_length"] = meta.max_length
            kwargs["pattern"] = meta.pattern
            kwargs["lower_case"] = meta.to_lower
            kwargs["upper_case"] = meta.to_upper
        # forward other metadata
        else:
            field_meta.append(meta)

    if json_schema_extra:
        kwargs["schema_extra"] = json_schema_extra

    kwargs = {k: v for k, v in kwargs.items() if v is not None}

    if kwargs:
        kwarg_definition = ParameterKwarg(**kwargs)
        field_meta.append(kwarg_definition)

    if field_meta:
        field_definition_kwargs["raw"] = field_annotation
        for meta in field_meta:
            field_annotation = Annotated[field_annotation, meta]

    return FieldDefinition.from_annotation(
        annotation=field_annotation,
        **field_definition_kwargs,
    )


def get_model_info(
    annotation: Any,
    prefer_alias: bool = False,
) -> PydanticModelInfo:
    model: type[pydantic_v1.BaseModel | pydantic_v2.BaseModel]  # pyright: ignore[reportInvalidTypeForm,reportGeneralTypeIssues]

    if is_generic(annotation):
        is_generic_model = True
        model = pydantic_unwrap_and_get_origin(annotation) or annotation
    else:
        is_generic_model = False
        model = annotation

    if is_pydantic_2_model(model):
        model_config = model.model_config
        model_field_info = model.model_fields
        title = model_config.get("title")
        example = model_config.get("example")
        is_v2_model = True
    else:
        model_config = model.__config__  # type: ignore[assignment, union-attr]
        model_field_info = model.__fields__  # type: ignore[assignment]
        title = getattr(model_config, "title", None)
        example = getattr(model_config, "example", None)
        is_v2_model = False

    model_fields: dict[str, pydantic_v1.fields.FieldInfo | pydantic_v2.fields.FieldInfo] = {  # pyright: ignore[reportInvalidTypeForm,reportGeneralTypeIssues]
        k: getattr(f, "field_info", f) for k, f in model_field_info.items()
    }

    if is_v2_model:
        # extract the annotations from the FieldInfo. This allows us to skip fields
        # which have been marked as private
        # if there's a default factory, we wrap the field in 'NotRequired', to signal
        # that it is not required
        model_annotations = {
            k: NotRequired[field_info.annotation] if field_info.default_factory else field_info.annotation  # type: ignore[union-attr]
            for k, field_info in model_fields.items()
        }

    else:
        # pydantic v1 requires some workarounds here
        model_annotations = {
            k: f.outer_type_  # type: ignore[union-attr]
            if f.required or f.default  # type: ignore[union-attr]
            else (NotRequired[f.outer_type_] if f.default_factory else Optional[f.outer_type_])  # type: ignore[union-attr]
            for k, f in model.__fields__.items()  # type: ignore[union-attr]
        }

    if is_generic_model:
        # if the model is generic, resolve the type variables. We pass in the
        # already extracted annotations, to keep the logic of respecting private
        # fields consistent with the above
        model_annotations = pydantic_get_type_hints_with_generics_resolved(
            annotation, model_annotations=model_annotations, include_extras=True
        )

    create_field_definition: _CreateFieldDefinition = (
        _create_field_definition_v2 if is_v2_model else _create_field_definition_v1  # type: ignore[assignment]
    )

    property_fields = {
        field_info.alias if field_info.alias and prefer_alias else k: create_field_definition(
            field_annotation=model_annotations[k],
            name=field_info.alias if field_info.alias and prefer_alias else k,
            default=Empty
            if is_undefined_sentinel(field_info.default) or is_pydantic_undefined(field_info.default)
            else field_info.default,
            field_info=field_info,
        )
        for k, field_info in model_fields.items()
    }

    computed_field_definitions = create_field_definitions_for_computed_fields(
        model,
        prefer_alias=prefer_alias,
    )
    property_fields.update(computed_field_definitions)

    return PydanticModelInfo(
        pydantic_version="2" if is_v2_model else "1",
        title=title,
        example=example,
        field_definitions=property_fields,
        is_generic=is_generic_model,
        model_fields=model_fields,
    )



import pytest
from click.testing import CliRunner

from litestar.cli.main import litestar_group as cli_command


@pytest.mark.parametrize("option", ["--version", "-V"])
def test_suggest_version(option: str, runner: CliRunner) -> None:
    result = runner.invoke(cli_command, option)

    assert "Did you mean command `version`?" in result.output



from __future__ import annotations

from contextlib import AsyncExitStack
from typing import Any, AsyncGenerator, Iterable

from psycopg import AsyncConnection
from psycopg.sql import SQL, Identifier

from litestar.channels.backends.base import ChannelsBackend


class PsycoPgChannelsBackend(ChannelsBackend):
    _listener_conn: AsyncConnection[Any]

    def __init__(self, pg_dsn: str) -> None:
        self._pg_dsn = pg_dsn
        self._subscribed_channels: set[str] = set()
        self._exit_stack = AsyncExitStack()

    async def on_startup(self) -> None:
        self._listener_conn = await AsyncConnection[Any].connect(self._pg_dsn, autocommit=True)
        await self._exit_stack.enter_async_context(self._listener_conn)

    async def on_shutdown(self) -> None:
        await self._exit_stack.aclose()

    async def publish(self, data: bytes, channels: Iterable[str]) -> None:
        dec_data = data.decode("utf-8")
        async with await AsyncConnection[Any].connect(self._pg_dsn, autocommit=True) as conn:
            for channel in channels:
                await conn.execute(SQL("NOTIFY {channel}, {data}").format(channel=Identifier(channel), data=dec_data))

    async def subscribe(self, channels: Iterable[str]) -> None:
        for channel in set(channels) - self._subscribed_channels:
            await self._listener_conn.execute(SQL("LISTEN {channel}").format(channel=Identifier(channel)))
            self._subscribed_channels.add(channel)
        await self._listener_conn.commit()

    async def unsubscribe(self, channels: Iterable[str]) -> None:
        for channel in channels:
            await self._listener_conn.execute(SQL("UNLISTEN {channel}").format(channel=Identifier(channel)))
            await self._listener_conn.commit()

        self._subscribed_channels = self._subscribed_channels - set(channels)

    async def stream_events(self) -> AsyncGenerator[tuple[str, bytes], None]:
        async for notify in self._listener_conn.notifies():
            yield notify.channel, notify.payload.encode("utf-8")

    async def get_history(self, channel: str, limit: int | None = None) -> list[bytes]:
        raise NotImplementedError()



from __future__ import annotations

from typing import TYPE_CHECKING

from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column

from litestar import Litestar, post
from litestar.dto import dto_field
from litestar.plugins.sqlalchemy import SQLAlchemySerializationPlugin

if TYPE_CHECKING:
    from typing import List


class Base(DeclarativeBase): ...


class TodoItem(Base):
    __tablename__ = "todo_item"
    title: Mapped[str] = mapped_column(primary_key=True)
    done: Mapped[bool]
    super_secret_value: Mapped[str] = mapped_column(info=dto_field("private"))


@post("/", sync_to_thread=False)
def add_item(data: TodoItem) -> List[TodoItem]:
    data.super_secret_value = "This is a secret"
    return [data]


app = Litestar(route_handlers=[add_item], plugins=[SQLAlchemySerializationPlugin()])



from typing import Any, Dict

from litestar import Litestar, get
from litestar.datastructures import State


class MyState(State):
    count: int = 0

    def increment(self) -> None:
        self.count += 1


@get("/", sync_to_thread=False)
def handler(state: MyState) -> Dict[str, Any]:
    state.increment()
    return state.dict()


app = Litestar(route_handlers=[handler])
